'''
Created on Nov 20, 2013

@author: kowitz
'''
import logging
from numpy.core.numeric import zeros, array
from numpy.linalg.linalg import lstsq, eigvals
from numpy import dot, mgrid
import numpy as np
from numpy.ma.core import conjugate

from geneCombinationGrid import geneEVOpticomGridEigenvector, geneCombinationGridEigenvector,EVOpticomCombigridBase,IterativeRefinementEVOpticomGridEigenvector,DataProviderGene,DataProviderCP
from GeneInterface import geneEnvironment2, storeInCheckpoint,checkpoint2Local
from numpy.matlib import rand
import sys
from scipy.optimize.optimize import fmin
from scipy.optimize import fmin_cobyla, fmin_slsqp, fmin_cg
import scipy.optimize as opt
import scipy.linalg as scipy_linalg
from scipy.ndimage import zoom
from copy import copy
import os


def get_existing_checkpoints(dir, combiGrid, combiGridConstructor, scheme=None):
    """
    Convenience Function to fill a combination grid with the right eigenvectors!
    Get the existing checkpoints in a directory and find the ones with the eigenvectors belonging to each other.
    That is using the hermitian angle of the eigenvectors
    :param dir: directory containing all the checkpoint directories
    :param combiGrid: combineGrids instance
    :param combiGridConstructor: function handle for creating the combination grid eigenvector (!) instance
    :param scheme: combination scheme
    """

    existing_checkpoints = os.listdir(dir)
    for folderName in existing_checkpoints:

        folder = os.path.join(dir, folderName) + '/'

        if os.path.isdir(folder) and ('_checkpoint' in folder.split('/')[-2]):
            log.debug('Found folder: ' + folder)

            try:
                cp = checkpoint2Local(folder)
                level = cp.getLevel()
            except IOError:
                log.debug('not a checkpoint folder')
                continue

            if scheme != None:
                keys = scheme.dictOfScheme.keys()
                keys.append(scheme.getMaxTupel())
                if not level in keys:
                    log.debug('did not find key, proceeding to next grid')
                    continue

            provider = DataProviderCP(cp, useMapProxies=True)

            if len(combiGrid.gridsDict) == 0:
                combiGrid.addGrid(combiGridConstructor(level, provider, order=order))
                grid = geneCombinationGridEigenvector(level, provider, order=order)
                reference = grid.extrapolateData(level).flatten()
                reference /= np.linalg.norm(reference, 2)

                referenceLevel = level
                log.debug(referenceLevel)
            else:
                # #find appropriate eigenvector
                nEV = provider.cp.getPar('n_ev')
                foundAngles = []
                for i in range(nEV):
                    grid = geneEVOpticomGridEigenvector(level, provider, nEV=i, order=order)
                    current = grid.extrapolateData(referenceLevel).flatten()
                    current /= np.linalg.norm(current, 2)
                    log.debug(current.shape)
                    log.debug(reference.shape)
                    cosAlpha = np.vdot(reference, current)
                    foundAngles.append(abs(cosAlpha))
                    log.debug('Hermitescher Winkel ' + str(abs(cosAlpha)))

                foundAngles = np.array(foundAngles)
                maxInd = foundAngles.argmax()

                combiGrid.addGrid(combiGridConstructor(level, provider, nEV=maxInd, order=order))
                log.info('Added Eigenvector ' + str(maxInd) + ' of level' + str(level) + ' to combigrid')



class combineGrids(object):
    '''
    a class to combine several grids
    '''


    def __init__(self, combischeme):
        '''
        Constructor
        TODO it has to be changed to have the scheme optional
        '''

        self.log = logging.getLogger(__name__)
        self.combinationScheme = combischeme
        self.getSchemeDict = self.combinationScheme.getCombinationDictionary
        #         self.combischeme = {}
        #         levels = self.scheme.getLevels()
        #         coeff = self.scheme.getCoef()
        #         for i in range(self.scheme.getNrSapcescombischeme()):
        #             self.combischeme[levels[i]] = coeff[i]

        self.gridsDict = {}

        self.log.debug('created single combigrid')

        self.maxSpace = None

    def getMaxSpace(self):
        if self.maxSpace is None:
            buffer = [0 for i in range(self.combinationScheme.getDim())]
            for k in self.getSchemeDict():
                for i in range(self.combinationScheme.getDim()):
                    if k[i] > buffer[i]:
                        buffer[i] = k[i]
            return tuple(buffer)
        else:
            return self.maxSpace

    def setMaxSpace(self,max_space,auto = False):
        #TODO check if its really fitting current scheme
        if auto:
            self.maxSpace=None
        else:
            self.maxSpace = max_space

        self.log.debug('Set fixed maxspace to '+ str(max_space))

    def getCombiScheme(self):
        return self.combinationScheme

    def setCombiScheme(self,combischeme):
        self.setMaxSpace(combischeme.getMaxTupel())
        self.combinationScheme = combischeme
        self.getSchemeDict = self.combinationScheme.getCombinationDictionary

    def getMaxGrid(self):
        l = array(self.getMaxSpace())
        return 2 ** l + 1

    def getSchemeDict(self):
        return self.getSchemeDict()

    def addGrid(self, grid, index=None):
        if self.gridsDict != {}:
            t = type(grid)
            if type(self.gridsDict.values()[0]) != t:
                raise TypeError('new gridType is not the same as the other ones')

        level = tuple(grid.levelvec)

        key = (level,) if index is None else (level, index)

        self.gridsDict[key] = grid
        self.log.info('Added grid with key ' + str(key))

    def addGrids(self, gridIterable):
        for grid in gridIterable:
            self.addGrid(grid)

    def missingGrids(self):
        setOfCombiLevels = set()
        for k in self.getSchemeDict():
            if self.getSchemeDict()[k] != 0.:
                setOfCombiLevels.add(k)
        if setOfCombiLevels.issubset(self.get_levels_in_dict()):
            return []
        else:
            return list(setOfCombiLevels.difference(self.get_levels_in_dict()))

    def get_levels_in_dict(self):
        self.levels_in_dict = set()
        for key in self.gridsDict.keys():
            self.levels_in_dict.add(key[0])
        return self.levels_in_dict


    def missingGridsDownSet(self):
        setOfCombiLevels = set()
        setOfCombiLevels = self.combinationScheme.getUnifiedDownSet()

        if setOfCombiLevels.issubset(self.get_levels_in_dict()):
            return []
        else:
            return list(setOfCombiLevels.difference(self.get_levels_in_dict()))

    def isSchemeComplete(self):
        if self.missingGrids() == []:
            return True
        else:
            return False

    def getCombination(self,normalization=None,*args):
        if normalization is not None:
                if normalization=='all':
                        my_norm=lambda x: np.linalg.norm(x.flatten(),ord=2)
                else: #if normalization=='abs':
                        my_norm=lambda x: np.linalg.norm(abs(x.flatten()),ord=2)
        else:
                my_norm = lambda x:1.0

        test = self.isSchemeComplete()

        if test != True:
            raise AssertionError('The levelvector(s) ' + str(test) + ' are missing')

        #extrapolatedSize = self.gridsDict[(self.getSchemeDict().keys()[0],)].getExtrapolatedDataSize(self
        # .getMaxSpace())
        #self.log.debug(extrapolatedSize)
        result = None  # zeros(extrapolatedSize,dtype=complex)
        for li in self.getSchemeDict():
            self.log.info('Current Grid to be added to combination: {}'.format(li))
            l = (li,)
            if self.getSchemeDict()[li] != 0.:
                if result is None:
                    if(len(args)==0):
                        maxSpace = self.getMaxSpace()
                    else:
                        maxSpace = args[0]
                    #self.log.debug(self.gridsDict[l].extrapolateData(maxSpace).shape)
                    data = self.gridsDict[l].extrapolateData(maxSpace)
                    result =      data  * self.getSchemeDict()[li]/my_norm(data)
                    #self.log.debug(result.shape)
                else:
                    data = self.gridsDict[l].extrapolateData(maxSpace)
                    result += data* self.getSchemeDict()[li]/my_norm(data)

        self.log.info('combined all grids')
        return result

    def getSparseGridPoints(self):
        test = self.isSchemeComplete()

        if test != True:
            raise AssertionError('The levelvector(s) ' + str(test) + ' are missing')

        sgPoints = set()# zeros(extrapolatedSize,dtype=complex)
        for li in self.getSchemeDict():
            l = (li,)
            if self.getSchemeDict()[li] != 0.:
                gridShape = self.gridsDict[l].data.shape[1:]
                ind = []
                for sh in gridShape:
                    ind.append(slice(0,1,1j*sh))
                combiPoints = zip(*(x.flat for x in mgrid[ind]))
                sgPoints.update(combiPoints)

        return sgPoints

    def plotSparseGrid(self,size=(5,5), ptsize=8, color='b'):
        import matplotlib.pyplot as plt
        sgPoints = self.getSparseGridPoints()
        plt.figure(figsize=size)
        plt.scatter(*zip(*sgPoints),s=ptsize, color=color)
        plt.axis('equal')
        plt.axis('off')
        return plt.gcf()

    def plotFullGrid(self,size=(5,5), ptsize=8, color='b'):
        import matplotlib.pyplot as plt
        l = 2**array(self.getMaxSpace())+1
        ind = []
        for sh in l: 
            ind.append(slice(0,1,1j*sh))
        fgPoints = zip(*(x.flat for x in mgrid[ind]))
        plt.figure(figsize=size)
        plt.scatter(*zip(*fgPoints),s=ptsize, color=color)
        plt.axis('equal')
        plt.axis('off')
        return plt.gcf()

    def fillAllData(self):
        # for l in self.getSchemeDict():
        #     self.gridsDict[(l,)].fillData()
        for g in self.gridsDict.values():
            if g.levelvec in self.getSchemeDict():
                g.fillData()

    def getRelevantGridSizes(self):
        #         return self.getDownSet()

        s = set([])
        self.log.debug(str(self.combinationScheme.getCombinationDictionary()))
        for g in self.combinationScheme.getCombinationDictionary().iterkeys():
            if self.combinationScheme.getCombinationDictionary()[g] != 0:
                s.add((g,))
        return s

    def getDownSet(self):
        s = set([])
        return self.combinationScheme.getUnifiedDownSet()

    def findOutliers(self,method='fdr_by',alpha=0.05,test_val=1e-2):
        from sets import Set
        from statsmodels.formula.api import ols
        from HelperFunctions import is_outlier

        # First, if there is a fault on the highest subspace, we try to identify
        # it by observing if the surpluses decrease or increase.
        dim = self.combinationScheme.getDim()
        outlierGrids = Set()
        for key,grid in self.gridsDict.iteritems():
            sh = grid.gridSizes
            di = self.combinationScheme.keysPerLevel((1,)*dim,key[0])
            dictOfSurplusPerLevel = {}
            for l1,subs in di.iteritems():
                subs_shape = np.amax(subs,axis=0)
                dictOfSurplusPerLevel[l1] = []
                for s in subs:
                    dictOfSurplusPerLevel[l1] += list(np.abs(grid.getHierSubspace(s,'h').flatten()))
                dictOfSurplusPerLevel[l1] = np.max(array(dictOfSurplusPerLevel[l1]))
            # Check for highest subspace outlier
            maxSub = np.max(dictOfSurplusPerLevel.keys())
            diffSub = 3
            if dictOfSurplusPerLevel[maxSub] > dictOfSurplusPerLevel[maxSub-diffSub]:
                outlierGrids.add(key[0])

        # Now we check all other subspaces
        # For all subspaces in the combination space, gather all versions of the subspace
        allSurplusesPerSub = {}
        for key,grid in self.gridsDict.iteritems():
            hierSubs = self.combinationScheme.hierSubs((1,1),key[0])
            for hierSub in hierSubs:
                if not allSurplusesPerSub.has_key(hierSub):
                    allSurplusesPerSub[hierSub] = [grid.getHierSubspace(hierSub,'h')]
                else:
                    allSurplusesPerSub[hierSub].append(grid.getHierSubspace(hierSub,'h'))
        for sub,arrs in allSurplusesPerSub.iteritems():
            allSurplusesPerSub[sub] = np.array(arrs)
        # Look for outliers on every subspace
        for sub,surp in allSurplusesPerSub.iteritems():
            sh = np.array(np.shape(surp))
            if (sh == 1).any():
                continue
            surp = np.reshape(surp, (sh[0],-1))
                # If we have too few versions of a point (<=3), the outlier detection
                # is not trustworthy/robust
            if np.shape(surp)[0] <= 3:
                continue
            for s in xrange(np.shape(surp)[1]):
                current_point = surp[:,s]
                X = range(1,len(current_point)+1)
                #X = np.ones_like(current_point)
                regression = ols("data ~ x", data=dict(data=current_point, x=X)).fit()
                # Find outliers
                test = regression.outlier_test(method=method,alpha=alpha)
                outliers = [current_point[i] for i,t in enumerate(test.icol(2)) if t < test_val]
                # Takes care of the case where there is more than one outlier grid
                for out_i in xrange(len(outliers)):
                    ind = np.where(current_point==outliers[out_i])[0]
                    i = 0
                    for key,grid in self.gridsDict.iteritems():
                        if sub in self.combinationScheme.hierSubs((1,1),key[0]):
                            if i == ind:
                                outlierGrids.add(key[0])
                                break
                            else:
                                i = i+1
                        else:
                            continue
        return list(outlierGrids)


class combineGridsLS(combineGrids):
    def __init__(self, combischeme):
        combineGrids.__init__(self, combischeme)
        self.reference = self.getMaxSpace()
        self.log.debug(str(self.reference) + ' set as reference Grid')

    def getLSCombination(self,normalize=False,with_fault=False):
        assert self.isSchemeComplete()
        assert self.getMaxSpace() in self.get_levels_in_dict()
        l = self.getMaxSpace()
        requiredGrids = []
        item_iterator = []
        for item in self.combinationScheme.getCombinationDictionary().iteritems():
            if item[1] != 0:
                requiredGrids.append(item[0])
            item_iterator.append(item[0])
        listOfExtrapolated = [self.gridsDict[(t,)].extrapolateData(l).flatten() for t in requiredGrids]
        if with_fault:
                rand_i = np.random.randint(len(listOfExtrapolated))
                listOfExtrapolated[rand_i] = np.random.random(len(listOfExtrapolated[rand_i]))
        if normalize:
                for i in range(len(listOfExtrapolated)):
                        listOfExtrapolated[i]=listOfExtrapolated[i]/np.linalg.norm(listOfExtrapolated[i])
        A = array(listOfExtrapolated)
        b = self.gridsDict[(l,)].extrapolateData(l).flatten()
        if normalize:
                b/=np.linalg.norm(b)

        coeff = lstsq(A.T, b)[0]
        print 'coeff: ',coeff
        print 'item iterator',item_iterator
        combinedLSSolution = dot(A.T, coeff)
        self.LS_coeff = {}
        for item,coeff_item in zip(item_iterator,coeff):
                print item
                self.LS_coeff[item]=coeff_item
        g = list(self.getMaxGrid())
        g.insert(0, -1)
        return combinedLSSolution.reshape(g)


class combineGridsOpticomBase(combineGrids):
    def __init__(self, combischeme, dontUseScheme=False,reducedSystem=False):
        combineGrids.__init__(self, combischeme)
        self.log.debug('created ' + str(self.__class__))
        self.adjoint = lambda x: x.conjugate().T
        self.dontUseScheme = dontUseScheme
        self.reducedSystem = reducedSystem

    def getRelevantGridSizes(self):
        if self.dontUseScheme:
            #print '-------------------', self.gridsDict.keys()
            return self.gridsDict.keys()
        else:
            return combineGrids.getRelevantGridSizes(self)

    def addGrid(self, grid, **kwargs):
        assert isinstance(grid, EVOpticomCombigridBase)
        combineGrids.addGrid(self, grid, **kwargs)

    def getLG(self,ions_only=False,smoothing=False):
        relevantGrids = self.getRelevantGridSizes()
        referenceSize = self.getMaxSpace()

        G = []
        for g in relevantGrids:
            grid = self.gridsDict[g]

            if ions_only:
                # print 'ions only!'
                Gsingle = grid.applyOperator(referenceSize,smoothing=smoothing)[:1]
            else:
                Gsingle = grid.applyOperator(referenceSize,smoothing=smoothing)

            G.append(Gsingle.flatten())
            self.log.debug(Gsingle.shape)
        LG = np.array(G)
        self.log.debug(str(LG.shape))
        self.log.debug(str(G[0].shape))
        return LG.T

    def getG(self,ions_only=False):
        relevantGrids = self.getRelevantGridSizes()
        referenceSize = self.getMaxSpace()

        G = []
        for g in relevantGrids:
            grid = self.gridsDict[g]
            if ions_only:
                Gsingle = grid.extrapolateData(referenceSize)[:1]
            else:
                Gsingle = grid.extrapolateData(referenceSize)
            self.outcomeSize = Gsingle.shape
            G.append(Gsingle.flatten())
        G = np.array(G)
        self.log.debug(str(G.shape))
        return G.T

    def getResiduum(self):
        return np.linalg.norm(np.dot((self.LG-self.lamdaCombined*self.G),self.c/np.linalg.norm(self.c)))

    def getResidualVector(self):
        return (np.dot((self.LG-self.lamdaCombined*self.G),self.c/np.linalg.norm(self.c)))

    def getUntouchedResiduum(self):
        return (np.dot((self.LG-self.lamdaCombined*self.G),self.c))


    def getOriginalC(self):
        relevantGrids = self.getRelevantGridSizes()
        self.log.debug(str(relevantGrids))

        c = []
        for g in relevantGrids:
            c.append(self.combinationScheme.getCombinationDictionary()[g[0]])

        return np.array(c, dtype=complex)

    def getTurnedC(self):
        c = self.getOriginalC()
        self.setAllMatrices()
        ref = self.G[:, 0]
        n = self.G.shape[1]
        for i in range(1, n):
            c[i] *= (np.vdot(self.G[:, i], ref) / np.vdot(self.G[:, i], self.G[:, i]))

        return c

    def get_initialGuess(self):
        self.setAllMatrices()
        ref = self.G[:,0]
        n = self.G.shape[1]
        c=np.zeros(n,dtype=complex)
        for i in range(1, n):
            c[i] = (np.vdot(self.G[:, i], ref) / np.vdot(self.G[:, i], self.G[:, i]))
        c[0]=1.0
        return c

    def getResultFromUpdatedScheme(self, reshape=True):
        if reshape:
            return np.dot(self.getG(), self.c).reshape(self.outcomeSize)
        else:
            return np.dot(self.getG(), self.c)
            # erg = None
            # lmax = self.getMaxSpace()
            #
            # for c, g in zip(self.c, self.getRelevantGridSizes()):
            #     #             print c
            #     if erg == None:
            #         erg = c * self.gridsDict[g].extrapolateData(lmax)
            #     else:
            #         erg += c * self.gridsDict[g].extrapolateData(lmax)
            #     #         return np.dot(self.G,self.c)
            # return erg

    def setXS(self, x, s):
        self.x = x
        self.s = s

    def getSystemSize(self):
        return self.GG.shape

    def getNPartialSolutions(self):
        return self.GG.shape[0]


    def setSystem(self, lam):
        s = self.getSystemSize()
        self.system = np.zeros((s[0] + 1, s[0] + 1), dtype=complex)
        self.system[-1, :-1] = self.x
        self.system[:-1, -1] = self.s

        self.system[:-1, :-1] = self.getK(lam)

    def solveSystem(self, lam):
        self.setSystem(lam)
        b = np.zeros(self.getSystemSize()[0] + 1)
        b[-1] = 1
        res = np.linalg.solve(self.system, b)
        self.c = res[:-1]
        self.beta = res[-1]

    def optimize(self, lamr, lami):
        #for i in range(5):
        #    self.solveSystem(lamr + 1j * lami)
        #new_x = self.c
        #self.setXS(new_x,self.s)
        #print new_x
        #sys.exit()

        self.solveSystem(lamr + 1j * lami)
        # for i in range(5):
        #     new_x = np.linalg.solve(self.getK(lamr + 1j*lami),self.c)
        #     new_x /=np.linalg.norm(new_x,2)
        #     self.setXS(new_x,self.s)
        return abs(self.beta)#self.beta.conjugate() * self.beta

    #         return abs(self.beta)

    def _plotBeta(self, li, ax=None, res=20, box=0.75):
        import pylab as py

        X, Y = np.mgrid[li.real - box:li.real + box:res * 1j, li.imag - box:li.imag + box:res * 1j]
        beta_array = []
        for x, y in zip(X.flatten(), Y.flatten()):
            self.optimize(x, y)
            beta_array.append(self.beta)
        beta_array = np.array(beta_array).reshape((res, res))

        if ax == None:
            ax = subplot(111)
        #         py.contourf(X,Y,abs(beta_array),30)
        im = ax.pcolormesh(X, Y, abs(beta_array))
        py.colorbar(im, ax=ax)

    def computeOptimizedCoefficients(self, initial_lambda, precision=0.00001, ax=None, res=60, box=0.75, weights=None):
        '''Method of Osborne'''

        self.setAllMatrices(weights=weights)
        s = self.getSystemSize()
        self.log.debug(str(s))
        self.log.info('start optimization routine')

        li = initial_lambda

        #         x = np.random.rand(s[0]) + np.random.rand(s[0]) * 1j - 0.5 - 0.5j
        #         z = np.random.rand(s[0]) + np.random.rand(s[0]) * 1j - 0.5 - 0.5j
        if not hasattr(self, 'x'):
            x = z = np.ones(s[0])
            self.setXS(x, z)

        if ax != None:
            self._plotBeta(li, ax=ax, res=res, box=box)

        def func(lam):
            return self.optimize(lam[0], lam[1])

        #         self.solveSystem(li+10)

        lMin = fmin(func, np.array([li.real, li.imag]), xtol=precision)
        #         l=lMin[0]+1.j*lMin[1]
        self.lamdaCombined = lMin[0] + lMin[1] * 1.j
        self.log.info('Combined Lambda = ' + str(self.lamdaCombined))

        #print self.c
    def computeOptimizedCoefficientsOsborneOuterIteration(self, initial_lambda,precision=1e-6,smoothing=5,initial_x=None,weight=0.8):
        self.setAllMatrices()

        li = initial_lambda
        sysSize = self.getNPartialSolutions()
        if initial_x is None:
            y = np.ones(sysSize, dtype=complex)
        else:
            y = initial_x
        #s = np.ones(sysSize)
        s= y.conjugate()
        self.setXS(y,s)
        self.c = np.random.rand(sysSize)
        self.lamdaCombined=li

        # def funcOuter(x_vec):
        #     li = self.lamdaCombined
        #     def func(lam):
        #         return self.optimize(lam[0], lam[1])
        #
        # #         self.solveSystem(li+10)
        #
        #
        # # for i in range(smoothing):
        #     self.setXS(x_vec[:len(x_vec)/2]+1.0j*x_vec[len(x_vec)/2:],self.s)
        #     lMin = fmin(func, np.array([li.real, li.imag]), xtol=precision)
        #     #         l=lMin[0]+1.j*lMin[1]
        #     self.lamdaCombined = lMin[0] + lMin[1] * 1.j
        #     dMdl = self.lamdaCombined*self.GG - self.GLG
        #     #new_x = np.linalg.solve(self.getK(self.lamdaCombined),np.dot(dMdl,self.c))
        #     #new_x = np.dot(dMdl,self.c) ##fuer reele Probleme keine Verbesserung
        #     # weight = 0.2
        #     # new_x = np.linalg.solve(self.getK(self.lamdaCombined),np.dot(dMdl,self.c))*weight + (1-weight)*self.x
        #     # self.setXS(new_x/np.linalg.norm(new_x),self.s)
        #     print self.lamdaCombined,abs(self.beta)
        #     li = self.lamdaCombined
        #     return abs(self.beta)
        #
        # x_min = opt.fmin(funcOuter,list(self.c.real)+list(self.c.imag),maxfun=500)

        def func(lam):
            return self.optimize(lam[0], lam[1])

        #         self.solveSystem(li+10)


        for i in range(smoothing):
            lMin = fmin(func, np.array([li.real, li.imag]), xtol=precision)
            #         l=lMin[0]+1.j*lMin[1]
            self.lamdaCombined = lMin[0] + lMin[1] * 1.j
            #dMdl = self.lamdaCombined*self.GG - self.GLG
            #new_x = np.linalg.solve(self.getK(self.lamdaCombined),np.dot(dMdl,self.c))
            #new_x = np.dot(dMdl,self.c) ##fuer reele Probleme keine Verbesserung
            ##version with damped newton towards correct solution
            # new_x = np.linalg.solve(self.getK(self.lamdaCombined),np.dot(dMdl,self.c))*weight + (1-weight)*self.x

            ## Das ist die Version wo einfach die aktuelle Schaetzung des rechten Eigenvektors genommen wird
            new_x = weight*self.c/np.linalg.norm(self.c) + (1.0-weight)*self.x
            #new_x = np.linalg.solve(self.getK(self.lamdaCombined),self.c)
            # new_x /= np.linalg.norm(new_x)
            ## Take QR to get new x
            # n = self.getSystemSize()[0]
            # M = self.LG-self.lamdaCombined*self.G
            # Q,R = np.linalg.qr(M)
            # R1= R[:n,:n]
            # Q1= Q[:,:n]
            # z = np.linalg.solve(R,self.x)
            # y = np.linalg.solve(R1,-np.dot(self.adjoint(Q1),np.dot(self.G,z)))
            # new_x = np.dot(self.adjoint(M),np.dot(Q1,y))*weight + (1.0-weight)*self.x
            # new_x/=np.linalg.norm(new_x)

            print '---------------------------------'
            print np.linalg.norm(self.getResiduum())
            print 'new_x',np.linalg.norm(new_x),new_x
            print
            print 'c:\t',self.c

            # self.setXS(new_x,self.s)
            self.setXS(new_x,new_x.conjugate())
            print self.lamdaCombined,abs(self.beta)
            li = self.lamdaCombined

        self.log.info('Combined Lambda = ' + str(self.lamdaCombined))


class combineGridsEVOpticomGalerkin(combineGridsOpticomBase):
    def setAllMatrices(self,weights=None):
        self.log.debug('Setting all Matrices for EVOpticom')
        self.LG = self.getLG()
        self.G = self.getG()

        #self.LGLG = np.dot(self.adjoint(self.LG), self.LG)
        self.GG = np.dot(self.adjoint(self.G), self.G)
        self.GLG = np.dot(self.adjoint(self.G), self.LG)
        #self.LGG = np.dot(self.adjoint(self.LG), self.G)


    def getK(self, lam):
        return self.GLG - lam * self.GG


class combineGridsEVOpticom(combineGridsOpticomBase):
    def setAllMatrices(self, weights=None,norm_input=False,ions_only=False,smoothing=False):
        self.log.debug('Setting all Matrices for EVOpticom')

        LG = self.getLG(ions_only=ions_only,smoothing=smoothing)
        G = self.getG(ions_only=ions_only)
	
		

        ##TODO test norming the data
        if norm_input:
            for i in range(G.shape[1]):
                norm = np.linalg.norm(G[:,i],2)
                G[:,i]/=norm
                LG[:,i]/=norm

        self.G = G
        self.LG=LG
        self.log.debug(G.shape)
        self.log.debug(G[0].shape)
        self.log.debug(LG.shape)

        if weights is None:
            self.LGLG = np.dot(self.adjoint(LG), LG)
            self.GG = np.dot(self.adjoint(G), G)
            self.GLG = np.dot(self.adjoint(G), LG)
            self.LGG = np.dot(self.adjoint(LG), G)
            self.log.debug('LGLG shape: ' + str(self.LGLG.shape))
        else:
            self.LGLG = np.dot(self.adjoint(LG), (weights * LG.T).T)
            self.GG = np.dot(self.adjoint(G), (weights * G.T).T)
            self.GLG = np.dot(self.adjoint(G), (weights * LG.T).T)
            self.LGG = np.dot(self.adjoint(LG), (weights * G.T).T)

    def getK(self, lam):
        return self.LGLG + lam * lam.conjugate() * self.GG - lam.conjugate() * self.GLG - lam * self.LGG


    ###Experimental optimizers------------------------------------------------------------------------------------------


    def computeOptimizedCoefficientsMinSigma(self, initial_lambda, precision=0.0001, ax=None, res=60, box=0.75, weights=None):
        '''Method of Osborne'''

        self.setAllMatrices(weights=weights)
        s = self.getSystemSize()
        self.log.debug(str(s))
        self.log.info('start optimization routine')

        li = initial_lambda
        self.c= np.ones(s[0])

        #         x = np.random.rand(s[0]) + np.random.rand(s[0]) * 1j - 0.5 - 0.5j
        #         z = np.random.rand(s[0]) + np.random.rand(s[0]) * 1j - 0.5 - 0.5j
        if not hasattr(self, 'x'):
            x = z = np.ones(s[0])
            self.setXS(x, z)

        if ax != None:
            import pylab as py

            X, Y = np.mgrid[li.real - box:li.real + box:res * 1j, li.imag - box:li.imag + box:res * 1j]
            beta_array = []
            for x, y in zip(X.flatten(), Y.flatten()):
                K = self.LG-(x+1.j*y)*self.G
                sigma = np.linalg.svd(K,compute_uv=False)[-1]
                print sigma,
                beta_array.append(sigma)
            beta_array = np.array(beta_array).reshape((res, res))

            if ax == None:
                ax = subplot(111)
            #         py.contourf(X,Y,abs(beta_array),30)
            im = ax.pcolormesh(X, Y, abs(beta_array))
            py.colorbar(im, ax=ax)



        def func(lam):
            K = self.LG-(lam[0]+1.j*lam[1])*self.G
            # print lam
            sigma =  np.linalg.svd(K,compute_uv=False)[-1]
            return sigma

        #         self.solveSystem(li+10)

        lMin = fmin(func, np.array([li.real, li.imag]), ftol=0.0001,xtol=precision)
        #         l=lMin[0]+1.j*lMin[1]
        self.lamdaCombined = lMin[0] + lMin[1] * 1.j
        self.log.info('Combined Lambda = ' + str(self.lamdaCombined))

        K = self.LG-self.lamdaCombined*self.G
        KK = np.dot(self.adjoint(K),K)
        for i in range(10):
            self.c = np.linalg.solve(KK,self.c)
            self.c /=np.linalg.norm(self.c)

            #print self.c


    def computeOptimizedCoefficientsLS(self, initial_lambda, precision=0.00001):
        import pylab as py

        self.setAllMatrices()
        s = self.getSystemSize()

        li = initial_lambda

        #         x = np.random.rand(s[0]) + np.random.rand(s[0]) * 1j - 0.5 - 0.5j
        #         z = np.random.rand(s[0]) + np.random.rand(s[0]) * 1j - 0.5 - 0.5j

        x = z = np.ones(s[0])
        mu = 100.
        self.setXS(x, z)
        cin = self.getOriginalC()
        init_vec = np.array([li.real, li.imag] + list(cin.real) + list(cin.imag))
        sizeC = (len(init_vec) - 2) / 2

        def func(vec):
            lamc = vec[0] + 1j * vec[1]
            c = vec[2:sizeC + 2] + 1j * vec[sizeC + 2:]
            mat = self.LGLG - lamc.conjugate() * self.LGG - lamc * self.GLG + lamc * lamc.conjugate() * self.GG
            out = np.dot(c.T.conjugate(), np.dot(mat, c)) + mu / 2. * (np.vdot(c, c) - 1.0) ** 2
            #             print out
            return abs(out)

        minVec = fmin(func, init_vec, ftol=0.0000000001, maxfun=1000000, maxiter=1000000)
        #         l=lMin[0]+1.j*lMin[1]
        lMin = minVec[0] + 1j * minVec[1]
        self.lamdaCombined = lMin
        self.c = minVec[2:]
        self.c = self.c[:len(self.c) / 2] + 1j * self.c[len(self.c) / 2:]
        self.beta = 0
        print self.c

    def computeOptimizedCoefficientsDas(self,li_initial=None, c_initial=None,precision=0.00001,iter=10,ions_only=False,smoothing=False):
        ''' Alg. 3.2 of paper (least distance formulation)'''
        self.setAllMatrices(ions_only=ions_only,smoothing=smoothing)
        # print 'start QR'
        ## QR
        B = self.G
        A = self.LG
        n = A.shape[1]
        BA = np.concatenate((B,A),axis=1)
        R_temp = np.linalg.qr(BA,'r')
        R11 = R_temp[:n,:n]
        R12 = R_temp[:n,n:]
        R22 = R_temp[n:,n:]

        # print 'start QZ'
        ##QZ
        from scipy.linalg import qz
        R0,R,Q,Z = qz(R12,R11,output='complex')

        ##helper matrices
        E = np.dot(R22,Z)
        C = np.dot(self.adjoint(E),E)

        ##initial eigenpair
        ew,ev =  scipy_linalg.eig(R0,R)
        # li=ew[0]
        # v=ev[:,0]

        li=li_initial if li_initial is not None else ew[0]
        v = c_initial if c_initial is not None else ev[:,0]

        if (li_initial is not None) and (c_initial is None):
            # v = np.linalg.solve((self.LG-li*self.G).T.conjugate().dot(self.LG-li*self.G),0)[:,0]
            # v= np.linalg.lstsq(self.LG-li*self.G,zeros((self.LG.shape[0],1)))[0][:,0]
            # buf_mat = (self.LG-li*self.G)
            # buf_mat2 = buf_mat.conjugate().T.dot(buf_mat)

            # buf_mat3 = np.zeros((buf_mat2.shape[0]+1,buf_mat2.shape[0]+1))
            # buf_mat3[:-1,:-1] = buf_mat2
            # buf_mat3[-1,:-1] = np.ones(buf_mat2.shape[0])
            # buf_mat3[:-1,-1] = np.ones(buf_mat2.shape[0])
            # v = np.linalg.solve(buf_mat3,0)[:-1]

            # v = np.linalg.solve(buf_mat2+np.ones(buf_mat2.shape),100*np.ones(buf_mat2.shape[0]))


            grid_list = list(self.getRelevantGridSizes())
            scheme_dict = self.getSchemeDict()
            for i_grid in range(len((grid_list))):
                v[i_grid] = scheme_dict[grid_list[i_grid][0]]

            v/=np.linalg.norm(v)



        v = np.dot( self.adjoint(Z),v)

        M = zeros((n+1,n+1),dtype=complex)
        rhs = zeros(n+1,dtype=complex)
        d= np.dot
        d_li=1.0
        for i in range(iter):
            # while(abs(d_li)>precision):
            R_l = R0-li*R
            # print eigvals(R_l)

            ## solve 3.19
            M[:n,:n]=np.dot(self.adjoint(R_l),R_l)
            M[n,:n] = self.adjoint(v)
            M[:n,n] = v

            rhs[:n] = -np.dot(C,v)
            rhs[n] = np.vdot(v,v)

            erg = np.linalg.solve(M,rhs)
            v_new = erg[:n]
            self.beta = -erg[n]

            v = v_new/np.linalg.norm(v_new)

            Rv = d(R,v)
            d_li = np.vdot(Rv,dot(R_l,v))/(np.vdot(Rv,Rv))
            # print d_li, li+d_li,abs(self.beta)
            # print v

            li += d_li
        # print li

        self.lamdaCombined=li
        self.c = np.dot(Z,v)

    def computeOptimizedCoefficientsDasFixedLambda(self,li_initial, c_initial=None,precision=0.00001,iter=10,weight=0.8,smoothing=False):
        ''' Alg. 3.2 of paper (least distance formulation) without any modification of li'''
        self.setAllMatrices(smoothing=smoothing)
        print 'start QR'
        ## QR
        B = self.G
        A = self.LG
        n = A.shape[1]
        BA = np.concatenate((B,A),axis=1)
        R_temp = np.linalg.qr(BA,'r')
        R11 = R_temp[:n,:n]
        R12 = R_temp[:n,n:]
        R22 = R_temp[n:,n:]

        # print 'start QZ'
        ##QZ
        from scipy.linalg import qz
        R0,R,Q,Z = qz(R12,R11,output='complex')

        ##helper matrices
        E = np.dot(R22,Z)
        C = np.dot(self.adjoint(E),E)

        ##initial eigenpair
        ew,ev =  scipy_linalg.eig(R0,R)
        # li=ew[0]
        # v=ev[:,0]

        li=li_initial if li_initial is not None else ew[0]
        v = c_initial if c_initial is not None else ev[:,0]

        if (li_initial is not None) and (c_initial is None):
            #v = np.linalg.solve((self.LG-li*self.G).T.conjugate().dot(self.LG-li*self.G),0)[:,0]
            #v= np.linalg.lstsq(self.LG-li*self.G,zeros((self.LG.shape[0],1)))[0][:,0]
            #buf_mat = (self.LG-li*self.G)
            #buf_mat2 = buf_mat.conjugate().T.dot(buf_mat)

            # buf_mat3 = np.zeros((buf_mat2.shape[0]+1,buf_mat2.shape[0]+1))
            # buf_mat3[:-1,:-1] = buf_mat2
            # buf_mat3[-1,:-1] = np.ones(buf_mat2.shape[0])
            # buf_mat3[:-1,-1] = np.ones(buf_mat2.shape[0])
            # v = np.linalg.solve(buf_mat3,0)[:-1]

            #v = np.linalg.solve(buf_mat2+np.ones(buf_mat2.shape),100*np.ones(buf_mat2.shape[0]))


            grid_list = list(self.getRelevantGridSizes())
            scheme_dict = self.getSchemeDict()
            for i_grid in range(len((grid_list))):
                v[i_grid] = scheme_dict[grid_list[i_grid][0]]

            v/=np.linalg.norm(v)

        v = np.dot(self.adjoint(Z),v)

        M = zeros((n+1,n+1),dtype=complex)
        rhs = zeros(n+1,dtype=complex)
        d= np.dot
        d_li=1.0
        for i in range(iter):
        # while(abs(d_li)>precision):
	    print 'iteration: ',i
            R_l = R0-li*R
            # print eigvals(R_l)

            ## solve 3.19
            M[:n,:n]=np.dot(self.adjoint(R_l),R_l)
            M[n,:n] = self.adjoint(v)
            M[:n,n] = v

            rhs[:n] = -np.dot(C,v)
            rhs[n] = np.vdot(v,v)

            erg = np.linalg.solve(M,rhs)
            v_new = erg[:n]
            self.beta = -erg[n]

            #v = v_new/np.linalg.norm(v_new)
	    v = (weight*v_new+(1-weight)*v)
	    v /=np.linalg.norm(v_new)

            Rv = d(R,v)
            d_li = np.vdot(Rv,dot(R_l,v))/(np.vdot(Rv,Rv))
            print d_li, li+d_li,abs(self.beta)
            print abs(v)

            #li += d_li
	    #print li

        self.lamdaCombined=li
        self.c = np.dot(Z,v)
    def computeOptimizedCoefficientsDas2(self,li_initial=None, c_initial=None,precision=0.00001,iter=10):
        self.setAllMatrices()
        ## QR
        B = self.G
        A = self.LG
        n = A.shape[1]
        BA = np.concatenate((B,A),axis=1)
        R_temp = np.linalg.qr(BA,'r')
        R11 = R_temp[:n,:n]
        R12 = R_temp[:n,n:]
        R22 = R_temp[n:,n:]

        ##QZ
        from scipy.linalg import qz
        R0,R,Q,Z = qz(R12,R11,output='complex')

        ##helpher matrices
        E = np.dot(R22,Z)
        C = np.dot(self.adjoint(E),E)

        ##initial eigenpair
        ew,ev =  scipy_linalg.eig(R0,R)
        # li=ew[0]
        # v=ev[:,0]

        li=li_initial if li_initial is not None else ew[0]
        v = c_initial if c_initial is not None else ev[:,0]

        M = zeros((n+1,n+1),dtype=complex)
        rhs = zeros(n+1,dtype=complex)
        rhs2 = zeros(n+1,dtype=complex)
        rhs3 = zeros(n+1,dtype=complex)
        d= np.dot
        d_li=1.0
        for i in range(iter):
        # while(abs(d_li)>precision):
            R_l = R0-li*R
            # print eigvals(R_l)

            ## solve 3.24
            M[:n,:n]=np.dot(self.adjoint(R_l),R_l)
            M[n,:n] = self.adjoint(v)
            M[:n,n] = v

            rhs[:n] = -np.dot(C,v)
            rhs[n] = np.vdot(v,v)

            rhs2[:n] = d(self.adjoint(R_l),d(R,v))

            rhs3[:n] = d(self.adjoint(R),d(R_l,v))

            erg1 = np.linalg.solve(M,rhs)
            erg2 = np.linalg.solve(M,rhs2)
            erg3 = np.linalg.solve(M,rhs3)

            ##3.29
            p = erg1[:n]-v
            q= erg2[:n]+erg3[:n]
            r=1.0j*(erg2[:n]-erg3[:n])

            def func(g_in,*args):
                g = np.array([1]+list(g_in))
                V = array(args).T
                #print V.shape,g.shape
                Rlv=d(R_l,v)
                Vg = d(V,g)
                return 1.0/np.linalg.norm(Vg)**2 * (\
                    np.linalg.norm(d(R_l,Vg))**2 \
                    - np.linalg.norm(np.vdot(d(R,Vg),d(R_l,Vg)))**2/(np.linalg.norm(d(R,Vg))**2)\
                                     +np.vdot(Vg,d(C,Vg)))

            min_vec = fmin(func,[1,1,1],np.array([v,p,q,r]))
            # print min_vec
            alpha=min_vec[0]
            beta = min_vec[1]
            gamma = min_vec[2]
            v_new = v+alpha*p+beta*q+gamma*r

            v = v_new/np.linalg.norm(v_new)

            Rv = d(R,v)
            d_li = np.vdot(Rv,dot(R_l,v))/(np.vdot(Rv,Rv))
            print d_li#, li+d_li,abs(self.beta)
            # print v
            li += d_li


        self.lamdaCombined=li
        self.c = np.dot(Z,v)

    def computeOptimizedCoefficientsDas0(self,li_initial=None, c_initial=None,precision=0.00001,iter=15):
        self.setAllMatrices()
        ## QR
        B = self.G
        A = self.LG
        n = A.shape[1]
        BA = np.concatenate((B,A),axis=1)
        R_temp = np.linalg.qr(BA,'r')
        R11 = R_temp[:n,:n]
        R12 = R_temp[:n,n:]
        R22 = R_temp[n:,n:]

        ##QZ
        from scipy.linalg import qz
        R0,R,Q,Z = qz(R12,R11,output='complex')

        ##helpher matrices
        E = np.dot(R22,Z)
        C = np.dot(self.adjoint(E),E)

        ##initial eigenpair
        ew,ev =  scipy_linalg.eig(R0,R)
        # li=ew[0]
        # v=ev[:,0]

        li=li_initial if li_initial is not None else ew[0]
        v = c_initial if c_initial is not None else ev[:,0]

        M = zeros((n+1,n+1),dtype=complex)
        rhs = zeros(n+1,dtype=complex)
        d= np.dot
        d_li=1.0
        for i in range(iter):
        # while(abs(d_li)>precision):
            R_l = R0-li*R
            # print eigvals(R_l)
            RRC = np.dot(self.adjoint(R_l),R_l)+C
            # self.beta = np.dot(self.adjoint(v),np.dot(RRC,v))/ np.vdot(v,v)
            self.beta = np.linalg.norm(np.dot(R_l,v))**2+np.dot(self.adjoint(v),np.dot(C,v))

            v = np.linalg.solve(RRC,self.beta*v)

            v/=np.linalg.norm(v)

            Rv = np.dot(R,v)
            d_li = np.vdot(Rv,np.dot(R_l,v))/np.vdot(Rv,Rv)
            print d_li
            # print v

            li += d_li

        self.lamdaCombined=li
        self.c = np.dot(Z,v)


    def computeOptimizedCoefficientsGradientDescent(self, initial_lambda):
        '''like for rectangular but with my own minimizer. Wird schwer da mit sich aenderndem x die oeberflaeche rauh
        wird'''

        self.setAllMatrices()

        li = initial_lambda
        sysSize = self.getNPartialSolutions()

        ##gradient descent

    def computeOptimizedCoefficientsExactPenaltyFunctionNewton(self, initial_lambda, initial_u=None, mu=10000000., \
                                                               weights=None):
        '''using the derivatives from jennings thesis'''
        self.setAllMatrices()

        if initial_u is None:
            u = np.ones(self.getNPartialSolutions(), dtype=complex)
        else:
            u = initial_u
        li = initial_lambda

        dLambda = 1.0
        while (abs(dLambda) > 1e-6):
            X = -self.LGG + li.conjugate() * self.GG - self.GLG + li * self.GG
            Xd = 2.0 * self.GG
            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG
            Y = MM + mu * (np.dot(self.adjoint(u), u) - 1.0) * np.eye(MM.shape[0], dtype=complex)

            Y1u = np.linalg.solve(Y, u)
            Y1Xu = np.linalg.solve(Y, np.dot(X, u))
            alpha = 2.0 * mu / (1. + 2. * mu * np.dot(self.adjoint(u), Y1u))
            Y1barXu = 0.5 * (Y1Xu - alpha * np.dot(Y1u, np.dot(self.adjoint(u), Y1Xu)))

            a = 1. / (np.dot(self.adjoint(u), np.dot(Xd, u)) - 4.0 * np.dot(self.adjoint(u), np.dot(X, Y1barXu)))
            b = -2.0 * a * Y1barXu

            delta = a * (
                np.dot(self.adjoint(u), np.dot(X, u)) - 2.0 * alpha * (
                    np.dot(self.adjoint(u), np.dot(X, Y1u))) * np.vdot(u,
                                                                       u))
            gamma = -u - delta * Y1Xu + alpha * (
                np.vdot(u, u) + delta * (np.dot(self.adjoint(u), np.dot(X, Y1u)))) * Y1u

            dLambda = delta
            du = gamma

            #print du

            li -= dLambda
            u -= du
            change = np.linalg.norm(np.array(([dLambda] + list(du))), 2)
            # u/=np.linalg.norm(u)
            print '-----------------------'
            print change, li
            print u
            print abs(u)
            print u.sum()
        self.lamdaCombined = li
        self.c = u

    def computeOptimizedCoefficientsExactPenaltyFunctionGradientBased(self, initial_lambda, initial_u=None,
                                                                      mu=10000000., \
                                                                      weights=None,ax=None,box=0.5,res=40):
        self.setAllMatrices(norm_input=True)

        if initial_u is None:
            u = np.ones(self.getNPartialSolutions(), dtype=complex) / np.sqrt(self.getNPartialSolutions())
        else:
            u = initial_u
        li = initial_lambda



        def func(vec_in):
            offset = (len(vec_in) - 2) / 2 + 2
            li = vec_in[0] + 1.0j * vec_in[1]
            x = vec_in[2:offset] + 1.0j * vec_in[offset:]
            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG
            # print (np.dot(self.adjoint(x), np.dot(MM, x)) + 0.5 * mu * (np.vdot(x, x) - 1.0) ** 2).real,
            return (np.dot(self.adjoint(x), np.dot(MM, x)) + 0.5 * mu * (np.vdot(x, x) - 1.0) ** 2).real

        def penalty(vec_in):
            li = vec_in[0] + vec_in[1] * 1.j
            u = array(vec_in[2:2 + self.getNPartialSolutions()]) + 1.0j * array(
                vec_in[2 + self.getNPartialSolutions():])
            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG

            # s= u
            # s=np.ones(self.getNPartialSolutions())
            # return abs((np.dot(self.adjoint(u),np.dot(MM,u))\
            #             / np.vdot(u,u)\
            # + mu*((np.vdot(s,u))-1.0)**2))

            s = u
            extra_penalty = 1e10 if li.real < 0.0 else 0.0

            return (abs(np.dot(self.adjoint(u), np.dot(MM, u))\
            # / np.vdot(u, u) \
                        + mu/2.0 * (np.vdot(s, u) - 1.0) ** 2))

        def grad(vec_in):
            offset = (len(vec_in) - 2) / 2 + 2
            li = vec_in[0] + 1.0j * vec_in[1]
            x = vec_in[2:offset] + 1.0j * vec_in[offset:]

            d = np.dot
            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG
            K = MM


            grad = 4.0 * (d(K, x) + mu * (np.vdot(x, x) - 1.0) * x)
            # grad = 2.0 * (d(K, x) + mu * (np.vdot(x, x) - 1.0) * x)

            # GK = -self.LGG + li.conjugate() * self.GG - self.GLG + li*self.GG
            # GK = 2.0 * (-self.LGG + li.conjugate()*self.GG)
            GK = 2.0 * (-self.GLG + li * self.GG)
            grad_l = d(self.adjoint(x), d(GK, x))

            return np.array([grad_l.real, grad_l.imag] + list(grad.real) + list(grad.imag)).real

        def H(li,x):
            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG

            H11= np.dot(self.adjoint(x),np.dot(self.GG,x))
            H12 = 4.0* np.dot(-self.GLG+li*self.GG,x)
            H21 = H12
            H22 = MM.T + np.dot(x,self.adjoint(x))
            H22 += np.eye(H22.shape[0])*np.vdot(x,x)

            mat = np.zeros(len(x)+1,len(x)+1)
            mat[0,0]=H11
            mat[1:,0]=H12
            mat[0,1:]=H21
            mat[1:,1:]=H22

            return mat


        vec0 = np.array([li.real, li.imag] + list(u.real) + list(u.imag))

        #vec_min = opt.fmin_ncg(func, vec0, grad,avextol=1e-5)
        # vec_min = opt.fmin_cg(func, vec0, grad)
        #vec_min = opt.fmin_bfgs(func, vec0)
        vec_min = opt.fmin_bfgs(func, vec0, grad,gtol=1e-9)
        #vec_min =( opt.anneal(func,vec0))

        print vec_min
        # print 'grad:',grad(vec_min)

        self.lamdaCombined=vec_min[0]+1.0j*vec_min[1]
        offset = (len(vec_min)-2)/2+2
        self.c = vec_min[2:offset]+1.0j*vec_min[offset:]

        if ax is not None:
            import pylab
            X,Y = np.mgrid[li.real-box:li.real+box:1j*res,li.imag-box:li.imag+box:1j*res]
            print X[:,0]
            print Y[0]
            Z = []
            for i in range(len(X.flatten())):
                in_vec = np.array([X.flatten()[i]]+[Y.flatten()[i]]+list(self.c.real)+list(self.c.imag))
                Z.append(func(in_vec))
            Z=np.array(Z).reshape((res,res))
            mesh=ax.pcolormesh(X,Y,Z)
            pylab.colorbar(mesh)

        # u -= grad * 0.0001
        # print mu/2.0*(np.vdot(u,u)-1)**2,grad
        # print 2.0*np.dot(u.conjugate(),Y)

        # sys.exit()

    def computeOptimizedCoefficientsExactPenaltyFunctionGradientBasedHuber(self, initial_lambda, initial_u=None,
                                                                      mu=10000000., \
                                                                      weights=None,ax=None,box=0.5,res=40,\
                                                                      delta=0.01,delta_l=10000):
        self.setAllMatrices(norm_input=True)

        if initial_u is None:
            u = np.ones(self.getNPartialSolutions(), dtype=complex) / np.sqrt(self.getNPartialSolutions())
        else:
            u = initial_u
        li = initial_lambda



        def func(vec_in):
            offset = (len(vec_in) - 2) / 2 + 2
            li = vec_in[0] + 1.0j * vec_in[1]
            x = vec_in[2:offset] + 1.0j * vec_in[offset:]

            def huber(a,d):
                return d**2*(np.sqrt(1.0+np.vdot(a,a)/delta)-1.0).real
            # MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG
            # return (np.dot(self.adjoint(x), np.dot(MM, x)) + 0.5 * mu * (np.vdot(x, x) - 1.0) ** 2).real
            v = np.dot(self.LG-li*self.G,x)
            return ((huber(v, delta))+ 0.5 * mu * (np.vdot(x, x) - 1.0) ** 2).real


        def grad(vec_in):
            offset = (len(vec_in) - 2) / 2 + 2
            li = vec_in[0] + 1.0j * vec_in[1]
            x = vec_in[2:offset] + 1.0j * vec_in[offset:]

            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG
            g = 1.0+np.dot(self.adjoint(x),np.dot(MM,x))/(delta**2)
            grad = (0.5 /np.sqrt(g)*np.dot(self.adjoint(x),MM)) + 2.0*mu*(np.vdot(x,x)-1.0)*x

            g_l = 1.0+np.dot(self.adjoint(x),np.dot(MM,x))/(delta_l**2)
            grad_l = - 0.5 /np.sqrt(g_l)*np.dot(self.adjoint(x),np.dot(self.LGG-li.conjugate()*self.GG,x))

            # d = np.dot
            # MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG
            # K = MM
            #
            #
            # grad = 4.0 * (d(K, x) + mu * (np.vdot(x, x) - 1.0) * x)
            # # grad = 2.0 * (d(K, x) + mu * (np.vdot(x, x) - 1.0) * x)
            #
            # # GK = -self.LGG + li.conjugate() * self.GG - self.GLG + li*self.GG
            # # GK = 2.0 * (-self.LGG + li.conjugate()*self.GG)
            # GK = 2.0 * (-self.GLG + li * self.GG)
            # grad_l = d(self.adjoint(x), d(GK, x))

            return np.array([grad_l.real, grad_l.imag] + list(grad.real) + list(grad.imag)).real

        vec0 = np.array([li.real, li.imag] + list(u.real) + list(u.imag))

        # vec_min = opt.fmin_ncg(func, vec0, grad,avextol=1e-9,maxiter=10000)
        # vec_min = opt.fmin_cg(func, vec0, grad,maxiter=10000)
        # vec_min = opt.fmin_bfgs(func, vec0,gtol=1e-7)
        # vec_min = opt.fmin_bfgs(func, vec0, grad,gtol=1e-9,maxiter=10000)

        trace = []
        for i in range(30):
            vec0-=0.05*grad(vec0)
            print (vec0)
            trace.append(vec0.copy())
        print '=========================='
        trace = np.array(trace)
        if ax is not None:
            ax.plot(trace[:,0],trace[:,1],'o-')
        print trace[:,0]


        vec_min = vec0
        # print vec_min
        print 'grad:',grad(vec_min)

        self.lamdaCombined=vec_min[0]+1.0j*vec_min[1]
        offset = (len(vec_min)-2)/2+2
        self.c = vec_min[2:offset]+1.0j*vec_min[offset:]



        # u -= grad * 0.0001
        # print mu/2.0*(np.vdot(u,u)-1)**2,grad
        # print 2.0*np.dot(u.conjugate(),Y)

        # sys.exit()

    def computeOptimizedCoefficientsExactPenaltyFunctionWithHessian(self,initial_lambda,initial_u=None,mu=100000):
        '''still buggy'''
        self.setAllMatrices()

        if initial_u is None:
            u = np.ones(self.getNPartialSolutions(), dtype=complex) / np.sqrt(self.getNPartialSolutions())
        else:
            u = initial_u
        li = initial_lambda

        def grad(li,x):
            # offset = (len(vec_in) - 2) / 2 + 2
            # li = vec_in[0] + 1.0j * vec_in[1]
            # x = vec_in[2:offset] + 1.0j * vec_in[offset:]

            d = np.dot
            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG
            K = MM
            # x = u.real
            # y = u.imag

            # mu_part = mu * (np.vdot(u, u).real - 1.0)
            # grad_x = 2.0 * ( d(K, x) + 1.0j * (d(K, y) - d(K.conjugate(), y)) + mu_part * x)
            # grad_y = 2.0 * ( d(K, y) + 1.0j * (d(K.conjugate(), x) - d(K, x)) + mu_part * y)

            grad = 4.0 * (d(K, x) + mu * (np.vdot(x, x) - 1.0) * x)
            # grad = 2.0 * (d(K, x) + mu * (np.vdot(x, x) - 1.0) * x)

            # GK = -self.LGG + li.conjugate() * self.GG - self.GLG + li*self.GG
            # GK = 2.0 * (-self.LGG + li.conjugate()*self.GG)
            GK = 2.0 * (-self.GLG + li * self.GG)
            grad_l = d(self.adjoint(x), d(GK, x))

            # print '-----------------------------------------'
            #print mu_part
            #print x
            #print y
            #print  d(K,y)
            #print grad_x
            #print grad_y
            # print grad
            # print x
            # print grad_l
            return np.array([grad_l] + list(grad) )

        def H(li,x):
            # offset = (len(vec_in) - 2) / 2 + 2
            # li = vec_in[0] + 1.0j * vec_in[1]
            # x = vec_in[2:offset] + 1.0j * vec_in[offset:]

            # x = xi.reshape((len(xi),-1))

            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG


            H11= np.dot(self.adjoint(x),np.dot(self.GG,x))
            H12 = 4.0* np.dot(-self.GLG+li*self.GG,x)
            H21 = H12.conjugate()
            xi = x.reshape(len(x),-1)
            H22 = MM.T + np.dot(xi,self.adjoint(xi))
            H22 += np.eye(H22.shape[0])*np.vdot(x,x)
            # H22 = 4.0*mu*np.dot(x,self.adjoint(x))
            # print 'bla',H22
            mat = np.zeros((len(x)+1,len(x)+1))
            mat[0,0]=H11
            mat[1:,0]=H12
            mat[0,1:]=H21
            mat[1:,1:]=H22

            return mat


        for i in range(5):
            du = -np.linalg.solve(H(li,u),grad(li,u))

            li +=du[0]
            u +=du[1:]
            print
            print li
            print u

        sys.exit()

    def computeOptimizedCoefficientsExactPenaltyFunction(self, initial_lambda, initial_u=None, mu=10000000., \
                                                         weights=None):

        self.setAllMatrices(weights=weights)

        if initial_u is None:
            u = np.ones(self.getNPartialSolutions())
        else:
            u = initial_u

        def penalty(vec_in):
            li = vec_in[0] + vec_in[1] * 1.j
            u = array(vec_in[2:2 + self.getNPartialSolutions()]) + 1.0j * array(
                vec_in[2 + self.getNPartialSolutions():])
            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG

            # s= u
            # s=np.ones(self.getNPartialSolutions())
            # return abs((np.dot(self.adjoint(u),np.dot(MM,u))\
            #             / np.vdot(u,u)\
            # + mu*((np.vdot(s,u))-1.0)**2))

            s = u
            extra_penalty = 1e10 if li.real < 0.0 else 0.0
            return (abs(np.dot(self.adjoint(u), np.dot(MM, u))\
                        + mu/2.0 * (np.vdot(s, u) - 1.0) ** 2))


        l_out = opt.fmin_powell(penalty, [initial_lambda.real, initial_lambda.imag] + list(u.real) + list(u.imag), \
                                maxfun=1000000, ftol=1e-4, xtol=1e-8)
        # l_out = opt.fmin_bfgs(penalty, [initial_lambda.real, initial_lambda.imag] + list(u.real) + list(u.imag), \
        #                         gtol=1e-5,maxiter=10000)
        # l_out = opt.fmin_bfgs(penalty, [initial_lambda.real, initial_lambda.imag] + list(u.real) + list(u.imag),gtol=1e-7)
        # l_out = opt.fmin_cg(penalty,[initial_lambda.real,initial_lambda.imag]+list(u.real)+list(u.imag)\
        # )


        ## too slow
        #l_out = opt.fmin_cg(penalty,[initial_lambda.real,initial_lambda.imag]+list(u.real)+list(u.imag))
        self.penaltyVal = penalty(l_out)
        self.lamdaCombined = l_out[0] + 1.0j * l_out[1]
        self.c = array(l_out[2:2 + self.getNPartialSolutions()]) + 1.0j * array(l_out[2 + self.getNPartialSolutions():])
        print 'function value: ', penalty(l_out)

    def computeOptimizedCoefficientsExactPenaltyFunctionFixedLambda(self, initial_lambda, initial_u=None, mu=10000.):
        self.setAllMatrices()
        if initial_u is None:
            u = np.ones(self.getNPartialSolutions())
        else:
            u = initial_u

        def penalty(vec_in):
            li = initial_lambda
            u = array(vec_in[:self.getNPartialSolutions()]) + 1.0j * array(vec_in[self.getNPartialSolutions():])
            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG

            s = u
            #s=np.ones(self.getNPartialSolutions())
            return (abs(np.dot(self.adjoint(u), np.dot(MM, u)) / np.vdot(u, u) + mu * (np.vdot(s, u) - 1.0) ** 2))

        l_out = opt.fmin_powell(penalty, list(u.real) + list(u.imag), maxfun=100000, ftol=1e-6, xtol=1e-6)

        ## too slow
        #l_out = opt.fmin_cg(penalty,[initial_lambda.real,initial_lambda.imag]+list(u.real)+list(u.imag))
        self.penaltyVal = penalty(l_out)
        self.lamdaCombined = initial_lambda
        self.c = array(l_out[:self.getNPartialSolutions()]) + 1.0j * array(l_out[self.getNPartialSolutions():])

    def computeOptimizedCoefficientsExactPenaltyFunctionFixedC(self, initial_u,initial_lambda=None, mu=10000.):
        self.setAllMatrices()
        if initial_lambda is None:
            initial_lambda = 1.0 +1.0j


        def penalty(vec_in):
            li = vec_in[0]+1.0j*vec_in[1]
            u = initial_u
            MM = self.LGLG - li.conjugate() * self.GLG - li * self.LGG + li.conjugate() * li * self.GG

            s = u
            #s=np.ones(self.getNPartialSolutions())
            return (abs(np.dot(self.adjoint(u), np.dot(MM, u)) \
                        # / np.vdot(u, u)
                        + mu * (np.vdot(s, u) - 1.0) ** 2))

        l_out = opt.fmin_powell(penalty, [initial_lambda.real] + [initial_lambda.imag], maxfun=100000, ftol=1e-6, xtol=1e-6)

        ## too slow
        #l_out = opt.fmin_cg(penalty,[initial_lambda.real,initial_lambda.imag]+list(u.real)+list(u.imag))
        self.penaltyVal = penalty(l_out)
        self.lamdaCombined = l_out[0]+1.0j*l_out[1]

    def computeOptimizedCoefficientsOsborneRectangularByPaper(self,initial_lambda):
	   ''' only for exactly solvable system'''
	   li=initial_lambda
	   G = self.G = self.getG()
	   LG = self.LG = self.getLG()
	   sysSize = G.shape[1] #self.getNPartialSolutions()
	   y = np.ones(sysSize,dtype=complex)
	   for i in range(5):
		   M = self.LG - li * G
		   Q,R = np.linalg.qr(M)
		   z = np.linalg.solve(R,y)
		   M_dagger = np.linalg.pinv(M)
		   #M_dagger = np.linalg.inv(M.T.conjugate().dot(M)).dot(M.T.conjugate())
		   print M_dagger.shape
		   print M_dagger
		   print M_dagger.max()
		   #dMdy = M_dagger.dot(G.dot(M_dagger)) #- M_dagger.dot(M_dagger.T.conjugate().dot(np.eye(sysSize)-M.dot(M_dagger)))
		   #dMdy = M_dagger.dot(G).dot(M_dagger) #- M_dagger.dot(M_dagger.T.conjugate().dot(np.eye(sysSize)-M.dot(M_dagger)))
		   dMdy = G




		   print z.shape,R.shape,Q.shape,dMdy.shape,z.shape
		   y = np.linalg.solve(R,Q.T.conjugate().dot(dMdy.dot(z)))
		   y_max_ind = y.argmax()
		   dl = z[y_max_ind]/y[y_max_ind]
		   li = li - dl
		   print dl
	   print li

    def computeOptimizedCoefficientsOsborneRectangularSimple(self, initial_lambda, precision=0.000000001):
        self.setAllMatrices()

        li = initial_lambda
        sysSize = self.getNPartialSolutions()
        y = np.ones(sysSize, dtype=complex) * 1. / sysSize

        def optimize(li_in):
            li = li_in[0] + 1.0j * li_in[1]
            M = self.LG - li * self.G
            MT = self.adjoint(M)
            MM = self.LGLG - li.conj() * self.GLG - li * self.LGG + li.conj() * li * self.GG

            x = np.ones(self.G.shape[0], dtype=complex)
            s = np.ones(sysSize, dtype=complex)
            y = np.ones(sysSize, dtype=complex)

            #inner loop
            for i in range(20):
                z = np.linalg.solve(MM, np.dot(MT, x))
                y = np.linalg.solve(MM, np.dot(MT, np.dot(-self.G, z)))
                x = np.dot(-self.G, y) / np.vdot(s, y)

            self.beta = 1.0 / np.vdot(s, z)
            self.c = self.beta * z

            return abs(self.beta)


        l_out = fmin(optimize, [li.real, li.imag], xtol=precision, ftol=precision)

        self.lamdaCombined = l_out[0] + 1.0j * l_out[1]

    def computeOptimizedCoefficientsOsborneRectangularOuterIteration(self, initial_lambda, precision=0.00000001):
        '''same as osborn rectangular but with an outer iteration for x'''

        self.setAllMatrices()

        li = initial_lambda
        sysSize = self.getNPartialSolutions()
        y = np.ones(sysSize, dtype=complex)
        s = np.ones(sysSize)
        self.c = np.random.rand(sysSize)

        def optimize(li_in, *args):
            y = args[0]
            s = self.c  #args[1]
            li = li_in[0] + 1.0j * li_in[1]
            M = self.getLG() - li * self.getG()
            MT = self.adjoint(M)
            MM = self.LGLG - li.conj() * self.GLG - li * self.LGG + li.conj() * li * self.GG
            Q1, R = np.linalg.qr(M)

            z = np.linalg.solve(R, y)
            # y = np.linalg.solve(R,np.dot(Q1.T.conj(),np.dot(-self.G,z)))
            # x = np.dot(Q1,y)/np.vdot(s,y)

            self.beta = 1.0 / np.vdot(s, z)
            #print self.beta
            self.c = self.beta * z

            return abs(self.beta)

        def get_new_y(li_in, y):
            li = li_in[0] + 1.0j * li_in[1]
            M = self.getLG() - li * self.getG()
            MT = self.adjoint(M)
            MM = self.LGLG - li.conj() * self.GLG - li * self.LGG + li.conj() * li * self.GG
            Q1, R = np.linalg.qr(M)

            z = np.linalg.solve(R, y)
            y_new = np.linalg.solve(R, np.dot(Q1.T.conj(), np.dot(-self.getG(), z)))
            return y_new / np.linalg.norm(y_new)


        for i in range(2):
            l_out = opt.fmin_powell(optimize, [li.real, li.imag], [y, s], xtol=precision, ftol=precision)
            y_new = get_new_y(l_out, y)
            y = y_new

        self.lamdaCombined = l_out[0] + 1.0j * l_out[1]



    def computeOptimizedCoefficientsOsborneInnerIteration(self, initial_lambda,precision=1e-6,smoothing=5):
        '''funkioniert nicht fuer real world'''
        self.setAllMatrices()

        li = initial_lambda
        sysSize = self.getNPartialSolutions()
        y = np.ones(sysSize, dtype=complex)
        s = np.ones(sysSize)
        self.setXS(y,s)
        self.c = np.random.rand(sysSize)
        self.lamdaCombined=li

        def optimize(lamr,lami):
            for i in range(smoothing):
                self.solveSystem(lamr + 1j * lami)
                print i,abs(self.beta),(self.c)/np.linalg.norm(self.c)
                self.setXS(self.c/np.linalg.norm(self.c),self.s)
            return abs(self.beta)#self.beta.conjugate() * self.beta

        def func(lam):
            return optimize(lam[0], lam[1])

        lMin = fmin(func, np.array([li.real, li.imag]), xtol=precision)
        self.lamdaCombined = lMin[0] + lMin[1] * 1.j
        ## Das ist die Version wo einfach die aktuelle Schaetzung des rechten Eigenvektors genommen wird
        new_x = self.c/np.linalg.norm(self.c)


        print 'new_x',new_x
        print self.lamdaCombined,abs(self.beta)
        li = self.lamdaCombined

        self.log.info('Combined Lambda = ' + str(self.lamdaCombined))

    def computeOptimizedCoefficientsInverseIteration(self,initial_lambda,smoothing=10,outer_it=10):
        self.setAllMatrices()
        li = initial_lambda
        sysSize = self.getNPartialSolutions()
        y = np.ones(sysSize, dtype=complex)
        s = np.ones(sysSize)
        self.setXS(y,s)
        self.c =y
        self.lamdaCombined=li

        for j in range(outer_it):
            A = self.getK(li)
            AA = np.dot(self.adjoint(A),A)
            for i in range(smoothing):
                new_c = np.linalg.solve(AA,self.c)
                self.c = new_c/np.linalg.norm(new_c)
                print j,i,self.c

            opt= lambda l: np.linalg.norm(np.dot(self.LG - (l[0]+1.j*l[1])*self.G,self.c))
            min_li = fmin(opt,[li.real,li.imag])
            li = min_li[0] + 1.j*min_li[1]


        self.lamdaCombined=li

    def computeOptimizedCoefficientsOsborneRectangular(self, initial_lambda, precision=0.00000001):

        self.setAllMatrices()

        li = initial_lambda
        sysSize = self.getNPartialSolutions()
        y = np.ones(sysSize, dtype=complex) * 1. / sysSize

        def optimize(li_in):
            li = li_in[0] + 1.0j * li_in[1]
            M = self.getLG() - li * self.getG()
            MT = self.adjoint(M)
            MM = self.LGLG - li.conj() * self.GLG - li * self.LGG + li.conj() * li * self.GG
            Q1, R = np.linalg.qr(M)

            x = np.ones(self.G.shape[0], dtype=complex)
            s = np.ones(sysSize, dtype=complex)
            y = np.ones(sysSize, dtype=complex)

            #inner loop
            for i in range(5):
                z = np.linalg.solve(MM, np.dot(MT, x))
                y = np.linalg.solve(R, np.dot(Q1.T.conj(), np.dot(-self.getG(), z)))
                x = np.dot(Q1, y) / np.vdot(s, y)

            self.beta = 1.0 / np.vdot(s, z)
            #print self.beta
            self.c = self.beta * z

            return abs(self.beta)


        l_out = fmin(optimize, [li.real, li.imag], xtol=precision, ftol=precision)

        self.lamdaCombined = l_out[0] + 1.0j * l_out[1]

    def computeOptimizedCoefficientL1Norm(self, initial_lambda, precision=0.00000000001):
        '''cobyla works, might be better with alternating technique, since the lambda is really far off'''
        self.setAllMatrices()

        li = initial_lambda

        #         cin = self.getOriginalC()
        cin = self.getTurnedC()

        init_vec = np.array([li.real, li.imag] + list(cin.real) + list(cin.imag))
        sizeC = (len(init_vec) - 2) / 2


        def func(vec):
            '''input vec - lambda.real,lambda,imag, c'''
            l = vec[0] + vec[1] * 1j
            out = np.linalg.norm(np.dot(self.LG - l * self.G, vec[2:sizeC + 2] + 1j * vec[sizeC + 2:]), 1)
            print l, out
            return np.linalg.norm(np.dot(self.LG - l * self.G, vec[2:sizeC + 2] + 1j * vec[sizeC + 2:]), 1)

        def funcPenalty(vec):
            l = vec[0] + vec[1] * 1j
            alpha = 1
            return np.linalg.norm(np.dot(self.LG - l * self.G, vec[2:sizeC + 2] + 1j * vec[sizeC + 2:]), 1) + abs(
                np.sum(vec[2:sizeC + 2] + 1j * vec[sizeC + 2:]) - 1.0) * alpha

        def funcPenalty2(vec):
            l = vec[0] + vec[1] * 1j
            alpha = 0.001
            #             print np.linalg.norm(np.dot(self.LG-l*self.G,vec[2:sizeC+2]+1j*vec[sizeC+2:]),1)+\
            #                 (np.vdot(vec[2:sizeC+2]+1j*vec[sizeC+2:],vec[2:sizeC+2]+1j*vec[sizeC+2:])-1.0)**2*alpha

            return (np.linalg.norm(np.dot(self.LG - l * self.G, vec[2:sizeC + 2] + 1j * vec[sizeC + 2:]), 1) + (np.vdot(
                vec[2:sizeC + 2] + 1j * vec[sizeC + 2:],
                vec[2:sizeC + 2] + 1j * vec[sizeC + 2:]) - 1.0) ** 2 * alpha).real

        def constraint1(vec):
            return (np.sum(vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j)).real - 1.0

        def constraint2(vec):
            return 1.0 - (np.sum(vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j)).real


        def constraint3(vec):
            return (np.sum(vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j)).imag

        def constraint4(vec):
            return -(np.sum(vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j)).imag

        def constraintAbs1(vec):
            return (np.vdot((vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j), (vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j))) - 1.0

        def constraintAbs2(vec):
            return -(
                np.vdot((vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j), (vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j))) + 1.0

        def eqconstraint1(vec):
            return (np.sum(vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j)).real

        def eqconstraint2(vec):
            return (np.sum(vec[2:sizeC + 2] + vec[sizeC + 2:] * 1j)).imag

        #         minVec = fmin(func,np.array([li.real,li.imag]+list(cin.real)+list(cin.imag)),xtol=precision,
        # maxfun=1000)
        self.log.info('start optimization')

        ##this works
        #         minVec = fmin_cobyla(func,init_vec\
        #                              , [constraint1,constraint2,constraint3,constraint4],rhobeg=0.1,disp=2,
        # rhoend=0.0001,maxfun=10000)
        minVec = fmin_cobyla(func, init_vec, [constraintAbs1, constraintAbs2], rhobeg=0.2, disp=2, rhoend=0.001,
                             maxfun=10000)
        #         minVec = fmin_slsqp(func, , [eqconstraint1,eqconstraint2]prime_ieqcons, args, iter, acc, iprint,
        # disp, full_output, epsilon)

        #         minVec = fmin(funcPenalty2, init_vec, maxfun=10000,maxiter=10000,xtol=precision)

        lMin = minVec[0] + 1j * minVec[1]
        self.c = minVec[2:]
        self.c = self.c[:len(self.c) / 2] + 1j * self.c[len(self.c) / 2:]

        print self.c
        print lMin

        self.lamdaCombined = lMin

        self.beta = 0


    def computeOptimizedCoefficientL1NormGradientDescent(self, initial_lambda, precision=0.00000000001):
        self.setAllMatrices()

        li = initial_lambda

        cin = self.getOriginalC()

        init_vec = np.array([li.real, li.imag] + list(cin.real) + list(cin.imag))
        offset = 2
        sizeC = (len(init_vec) - offset) / 2
        sC = sizeC + offset
        mu = 10.

        def penalizedFunction(vec):
            l = vec[0] + .1j * vec[1]
            c = vec[offset:sC] + 1.j * vec[sC:]
            return (np.linalg.norm(np.dot(self.LG - l * self.G, c), 1) + 0.5 * mu * (np.vdot(c, c) - 1.0) ** 2).real

        def gradient(vec):
            l = vec[0] + .1j * vec[1]
            c = vec[offset:sC] + 1.j * vec[sC:]

            ##gradient wrt lambda
            #B_i * c
            B = self.LG - l * self.G
            Bc = np.dot(B, c)
            mask = Bc > 0
            gradL = np.sum(np.dot(self.G[mask], c)) - np.sum(np.dot(self.G[~mask], c))

            ##gradient for c

            gradC = np.sum(B[mask], axis=0) - np.sum(B[~mask], axis=0)
            return np.array([gradL.real, gradL.imag] + list(gradC.real) + list(gradC.imag))

        self.log.info('start optimization')

        #         minVec = opt.fmin_bfgs(penalizedFunction,init_vec,gradient)

        lMin = minVec[0] + 1j * minVec[1]
        self.c = minVec[2:]
        self.c = self.c[:len(self.c) / 2] + 1j * self.c[len(self.c) / 2:]

        self.lamdaCombined = lMin
        self.beta = 0


    def computeOptimizedCoefficientL1NormAlternating(self, initial_lambda, precision=0.00000000001):
        self.setAllMatrices()

        li = initial_lambda

        cin = np.array(self.getOriginalC(), dtype=complex)

        init_vec = np.array(list(cin.real) + list(cin.imag))
        offset = 0
        sizeC = (len(init_vec) - offset) / 2
        sC = sizeC + offset
        mu = 10.0
        mu2 = 10000

        self.log.info('start optimization')


        def orig(c, l):
            return (np.linalg.norm(np.dot(self.LG - l * self.G, c), 1) + 0.5 * mu * (np.vdot(c, c) - 1.0) ** 2 + mu2 * (
                                                                                                                           c[
                                                                                                                               0].imag) ** 2).real

        def fOfc(vec, l):
            c = vec[offset:sC] + 1.j * vec[sC:]
            return orig(c, l)

        def fOfl(vec, c):
            l = vec[0] + 1j * vec[1]
            return orig(c, l)

        minC = init_vec

        for count in range(20):
            minC = opt.fmin(fOfc, minC, [li], xtol=0.01, maxiter=10000, maxfun=10000)
            cc = np.array(minC[offset:sC] + 1j * minC[sC:])
            print cc
            minL = opt.fmin(fOfl, [li.real, li.imag], [cc], xtol=0.00001, maxiter=10000, maxfun=10000)
            print minL
            li = minL[0] + 1j * minL[1]
        #         sys.exit()

        self.c = cc

        self.lamdaCombined = li
        self.beta = 0

    def computeOptimizedCoefficientL1Normcvxopt(self, initial_lambda, precision=0.00000000001):
        #         from  cvxopt import solvers,matrix,spmatrix, mul ,div
        import cvxopt as cvx
        from cvxopt.solvers import cpl

        self.setAllMatrices()

        li = initial_lambda

        cin = self.getOriginalC()

        init_vec = np.array([li.real, li.imag] + list(cin.real) + list(cin.imag))
        offset = 2
        sizeC = (len(init_vec) - offset) / 2
        sC = sizeC + offset
        mu = 1

        nu = len(cin) * 2
        nv = self.G.shape[0] * 2
        nx0 = nu + nv
        print nx0
        v = np.dot(self.LG - li * self.G, cin)
        x0 = cvx.matrix((list(cin.real) + list(cin.imag)) + list(v.real) + list(v.imag))


        def F(x=None, z=None):
            if x is None:
                return 1, x0
            ##f, Df
            u = x[:nu]
            uLong = cvx.matrix(np.zeros(nx0))
            uLong[:nu] = u
            #             uLong=x
            uNorm = (u.T * u - 1.0)
            f = cvx.matrix(mu * uNorm ** 2)
            Df = cvx.matrix(4.0 * uLong * uNorm).T
            if z is None:
                return f, Df
            H = cvx.matrix(uLong * uLong.T)
            for i in range(nu):
                H[i, i] = H[i, i] - 4.0
            return f, Df, H

        #         print (self.LG-li*self.G).real
        #         print cvx.matrix(self.LG).real()
        #         print cvx.matrix(((self.LG-li*self.G))).real()
        P = cvx.matrix([[cvx.matrix(self.LG - li * self.G).real(), -cvx.matrix(self.LG - li * self.G).imag()],
                        [cvx.matrix(self.LG - li * self.G).imag(), cvx.matrix(self.LG - li * self.G).real()]])
        c = cvx.matrix(np.zeros(nu + nv))
        c[nu:] = 1.0
        print c.size
        I = cvx.matrix(cvx.spmatrix(1.0, range(nv), range(nv)))
        print I.size
        print P.size
        G = cvx.matrix([[P, -1.0 * P], [-1.0 * I, -1.0 * I]])
        print G.size
        h = cvx.matrix(np.zeros(2 * nv))
        print h.size

        self.log.info('start optimization')

        retDict = cpl(c, F, G=G, h=h)
        print retDict
        sys.exit()

        lMin = minVec[0] + 1j * minVec[1]
        self.c = minVec[2:]
        self.c = self.c[:len(self.c) / 2] + 1j * self.c[len(self.c) / 2:]

        self.lamdaCombined = lMin
        self.beta = 0

    def computeOptimizedCoefficientL1Osborne(self, initial_lambda, precision=0.00000000001):
        ''' not workin'''
        self.setAllMatrices()

        li = initial_lambda

        #cin = self.getOriginalC()
        cin = np.ones(self.getSystemSize(), dtype=complex)
        beta_start = 0.0
        init_vec = np.array([li.real, li.imag] + [beta_start] + list(cin.real) + list(cin.imag))
        sizeC = (len(init_vec) - 3) / 2

        s = self.LG.shape
        system = np.zeros((s[0] + 1, s[1] + 1))
        system[:-1, -1] = np.ones(s[0])  #x
        system[-1, :-1] = np.ones(s[1])  # z
        c = np.zeros(s[1] + 1)
        b = np.zeros(s[0] + 1)
        b[-1] = 1

        def func(vec):
            '''input vec - lambda.real,lambda,imag, c'''
            l = vec[0] + vec[1] * 1j
            system[:-1, :-1] = self.LG - l * self.G
            c[:-1] = vec[3:sizeC + 3] + 1j * vec[sizeC + 3:]
            c[-1] = vec[2]
            return np.linalg.norm(np.dot(system, c) - b, 2)

        self.log.info('start optimization')
        minVec = fmin(func, init_vec, xtol=precision)

        lMin = minVec[0] + 1j * minVec[1]
        self.c = minVec[3:]
        self.c = self.c[:len(self.c) / 2] + 1j * self.c[len(self.c) / 2:]

        self.lamdaCombined = lMin
        self.beta = minVec[2]
        print lMin
        print self.beta

class combineGridsIterativeRefinementDummy(combineGridsEVOpticom):

    def setLargeOperator(self,getA):
        self.getA =getA

    @staticmethod
    def solve_schur(B,C,f,g,lam,grid,A_inverter):
        A1f = A_inverter(grid,f,lam)
        A1B = A_inverter(grid,B,lam)

        y = (g-np.vdot(C,A1f))/-np.vdot(C,A1B) ##auf vollem gitter?
        x = A1f - y*A1B
        return x,y

    def injection(self,x,lmax,s,full=True):
        '''only for 2**n+1 grids'''
        diff = array(lmax)-array(s)
        self.log.debug(diff)
        if full is True:
            for d in range(len(s)):
                self.log.debug('dim: '+str(d))
                for l in range(diff[d]):
                    slices=[slice(0,x.shape[i],1) for i in range(len(s))]
                    slices[d]=slice(0,x.shape[d],2)
                    buf = 0.5 * x[slices]
                    ##right neighbors
                    slices[d] = slice(1,x.shape[d],2)
                    s=list(buf.shape)
                    s[d]=1
                    self.log.debug(x[slices].shape)
                    buf += 0.25 * (np.concatenate((x[slices],zeros(s)),axis=d))
                    buf += 0.25 * (np.concatenate((zeros(s),x[slices]),axis=d))
                    ##boundary
                    slices[d]=slice(0,1)
                    buf[slices] *= 4./3.
                    slices[d]=slice(buf.shape[d]-1,buf.shape[d])
                    buf[slices] *=4./3.

                    x=buf
        else:
            slices=[slice(0,x.shape[i],2**diff[i]) for i in range(len(s))]
            x=x[slices]
        return x

    def get_shape(self,levels):
        b=array(self.boundaries).astype(int)*2-1
        return tuple(2**array(levels)+b)

    def refinement(self,iterations=10,fig=None,g_c_in=None,l_c_in=None,full_injection=True,l_list=[],g_list=[],useOpticom=True):
        if l_c_in is None:
            l_c = self.lamdaCombined
        else:
            l_c=l_c_in
        if g_c_in is None:
            g_c = self.getResultFromUpdatedScheme(reshape=True)
            g_c /=np.linalg.norm(g_c)
        else:
            g_c = g_c_in.copy()
        #r_c = self.getResidualVector().reshape(self.getMaxGrid())

        self.boundaries=self.gridsDict.values()[0].boundaries
        b=-(array(self.boundaries).astype(int)-1)
        slices = [slice(b[i],g_c.shape[i]-b[i],1) for i in range(len(g_c.shape))]

        A_large = self.getA(self.getMaxSpace())
        g_c_inner = g_c[slices]
        res_inner = (np.dot(A_large,g_c_inner.flatten())-l_c*g_c_inner.flatten()).reshape(g_c_inner.shape)
        r_c=zeros(self.getMaxGrid(),dtype=res_inner.dtype)
        r_c[1:-1,1:-1]=res_inner


        self.log.debug(str(g_c.shape)+'  '+str(r_c.shape))
        self.log.info("initial resdidual:\t{:.4e}".format(np.linalg.norm(r_c)))


        for i in range(iterations):
            self.log.debug('Start iterative refinement step '+str(i))
            d_g = np.zeros(g_c.shape,dtype=g_c.dtype)
            d_l = 0.0
            count=0
            dG = []
            dGi_orig=[]
            dL = []

            for g in self.getRelevantGridSizes():
                ## sample r_c and g_c
                g_c_sample = self.injection(g_c,self.getMaxSpace(),g[0],full=full_injection)
                r_c_sample = self.injection(r_c,self.getMaxSpace(),g[0],full=full_injection)


                self.log.debug(g_c_sample.shape)
                ## solve 2 systems for Schur komplement
                print 'norm g_c_sample', np.linalg.norm(g_c_sample),(g_c_sample.size*1.0)/g_c.size
                # A1rc = self.gridsDict[g].solveSystem(r_c_sample*g_c_sample.size/g_c.size,l_c).flatten()
                A1rc = self.gridsDict[g].solveSystem(r_c_sample*np.linalg.norm(g_c_sample),l_c).flatten()
                A1gc = self.gridsDict[g].solveSystem(g_c_sample,l_c).flatten()

                self.log.debug(A1rc.shape)

                ## solve using schur complement
                f2 = 0.0
                gs = g_c_sample.flatten()
                dl = (f2+np.vdot(gs,A1rc)) / (-np.vdot(gs,A1gc))
                dg = (A1rc + A1gc*dl).reshape(2**array(g[0])+1)

                if fig is not None:
                    count+=1
                    ax=fig.add_subplot(1,len(self.getSchemeDict()),count)
                    im=ax.pcolormesh(r_c_sample[1:-1,1:-1])
                    fig.colorbar(mappable=im)

                ## prolongate onto full grid
                zoom_factor=self.getMaxGrid()/(2**array(g[0])+1.)
                buf=zeros(self.getMaxGrid(),dtype=dg.dtype)
                if dg.dtype == complex:
                    buf += zoom(dg.real,zoom_factor,order=1)
                    buf += zoom(dg.imag,zoom_factor,order=1)*1.j
                else:
                    buf += zoom(dg,zoom_factor,order=1)
                dg=buf

                dG.append(dg[1:-1,1:-1].flatten())
                dL.append(dl)

                dGi_orig.append(dg)

                if not useOpticom:
                    ##das funktioniert irgendwie noch nicht
                    d_g += dg * self.combinationScheme.getCombinationDictionary()[g[0]]
                    d_l += dl * self.combinationScheme.getCombinationDictionary()[g[0]]

            ##opticom approach
            if useOpticom:
                nGrids = len(dG)
                dG=np.array(dG).T
                ## apply full operator
                LG = dG.copy()

                print 'shape:',dG.shape, self.getA(self.getMaxSpace()).shape
                for i in range(nGrids):
                    LG[:,i]=np.dot(self.getA(self.getMaxSpace()),dG[:,i])

                ## build matrix for solving
                K = LG.copy()

                K -= l_c*dG
                K -= np.dot(np.array([g_c[slices].flatten()]).T,np.array([dL]))
                KK = np.dot(K.conjugate().T,K)
                print 'shape kk',KK.shape,r_c.shape

                c = np.linalg.solve(KK,-np.dot(K.T,r_c[slices].flatten()))

                print 'combicoeff:\t',c

                for i in range(nGrids):
                    d_g+=c[i]*dGi_orig[i]
                    d_l+=c[i]*dL[i]
            #
            g_c += d_g
            l_c += d_l
            # g_c -= d_g
            # l_c -= d_l
            g_c_sample = g_c[slices]
            self.log.debug(g_c_sample.shape)

            buf = np.dot(self.getA(self.getMaxSpace()),g_c_sample.flatten()).reshape(g_c_sample.shape) - l_c*g_c_sample

            r_c = zeros(g_c.shape,dtype=buf.dtype)
            r_c[slices]=buf
            l_list.append([l_c,np.linalg.norm(r_c.flatten())])
            g_list.append(g_c.copy())
            print 'max',g_c.max()
            self.log.info("d_g: {:.3e}\tr_c: {:.3e}\td_l: {:.8f}".format(np.linalg.norm(d_g),np.linalg.norm(r_c),d_l))

        return g_c,l_c,r_c

class combineGridsIterativeRefinementBase(combineGridsEVOpticom):

    def refinement(self,iterations=10,g_c_in=None,l_c_in=None,a_tol='1e-7',r_tol='1e-10',log_dict=None):
        """
        Gene specific implementation
        """
        if log_dict is not None:
                log_dict['l']=[]
                log_dict['g']=[]
                log_dict['residual']=[]

        ##create reference grid
        geneEnv = self.gridsDict[list(self.getRelevantGridSizes())[0]].iterativeOperatorGeneEnv
        geneEnv_copy = geneEnvironment2( \
            copy(geneEnv.genePath), \
            copy(geneEnv.problemFolder),
            geneEnv.genePath+geneEnv.problemFolder+'output/placeholder_for_residual/')
        geneEnv_copy.parFileIO.updateParameter('read_checkpoint','T')
        geneEnv_copy.parFileIO.updateParameter('which_ev',"'rhs_only'")
        geneEnv_copy.runCommand=copy(geneEnv.runCommand)

        l_c = self.lamdaCombined                            if l_c_in is None else l_c_in
        g_c = self.getResultFromUpdatedScheme(reshape=True) if g_c_in is None else g_c_in

        g_c_norm = np.linalg.norm(g_c)
        # g_c_norm = g_c.size
        g_c /= g_c_norm

        storeInCheckpoint(g_c,geneEnv_copy.diagDir,chpt_name='checkpoint')
        geneEnv_copy.setGridSizeWithLevel(self.getMaxSpace())
        geneEnv_copy.parFileIO.updateFile()
        geneEnv_copy.runGENE()
        cpWithL = checkpoint2Local(geneEnv_copy.diagDir)
        providerWithL = DataProviderCP(cpWithL,useMapProxies=False)
        ref_grid = IterativeRefinementEVOpticomGridEigenvector(self.getMaxSpace(),providerWithL)
        d = ref_grid.extrapolateData(self.getMaxSpace())

        #TODO das sollte eigentlich auch mit DataProviderGene funktionieren...\
        # macht es aber nicht, da scheint wohl irgendwo ein Bug zu sein.

        r_c = d - l_c*g_c

        self.log.debug("g_c.shape:  {}\tr_c.shape: {}".format(g_c.shape,r_c.shape))

        print 'norm r_c\t',np.linalg.norm(r_c)
        print "{:+5e}\t{:+5e}\t".format(np.linalg.norm(d),np.linalg.norm(g_c))


        for i in range(iterations):
            hash_tag = str(np.linalg.norm(r_c))
            self.log.debug('Start of refinement step {}'.format(i))
            d_g = np.zeros(g_c.shape,dtype=complex)
            d_l = 0.0

            dG ={}
            dL ={}

            coef=1.0

            print '--------------------'

            for g in self.getRelevantGridSizes():
                ## sample r_c and g_c
                g_c_sample = self.gridsDict[g].injection(g_c)
                r_c_sample = self.gridsDict[g].injection(r_c)

                self.log.debug("{}\t{}".format(g_c_sample.shape,r_c_sample.shape))

                ## solve the 2 systems for Schur complement
                A1rc = self.gridsDict[g].solveSystem(r_c_sample*np.linalg.norm(g_c_sample.flatten()),
                                                     l_c,hash_tag,tag='r_c',a_tol=a_tol,r_tol=r_tol)
                # A1rc = self.gridsDict[g].solveSystem(r_c_sample*g_c_sample.size/g_c.size,
                #                                      l_c,hash_tag,tag='r_c')
                A1gc = self.gridsDict[g].solveSystem(g_c_sample,l_c,hash_tag,tag='g_c',a_tol=a_tol,r_tol=r_tol)

                ## solve using Schur-Complement
                f2 = 0.0
                gs = g_c_sample.flatten()
                dl = (f2 + np.vdot(gs,A1rc))/ (-np.vdot(gs,A1gc))
                dg = (A1rc + A1gc*dl).reshape(g_c_sample.shape)

                ## prolongation
                self.log.debug(np.linalg.norm(dg))
                dg = self.gridsDict[g].extrapolateData(self.getMaxSpace(),data_override=dg,no_proxy=True)
                self.log.debug(np.linalg.norm(dg))
                print "{}\t{:5e}\t{:5e}\t{:5e}\t{:5e}".format(g[0],np.linalg.norm(dg),dl,np.linalg.norm(A1gc),np.linalg.norm(g_c_sample*np.linalg.norm(g_c_sample)))

                coef = self.combinationScheme.getCombinationDictionary()[g[0]]

                dG[g]=dg
                dL[g]=dl
                # d_g += dg * coef
                # d_l += dl * coef

                # coef = 0.0

            dLG={}
            for g in self.getRelevantGridSizes():
                ## apply operator
                storeInCheckpoint(dG[g],geneEnv_copy.diagDir,chpt_name='checkpoint')
                geneEnv_copy.parFileIO.updateFile()
                geneEnv_copy.runGENE()
                cpLocal = checkpoint2Local(geneEnv_copy.diagDir)
                provLocal = DataProviderCP(cpLocal,useMapProxies=False)
                grid = IterativeRefinementEVOpticomGridEigenvector(self.getMaxSpace(),provLocal)
                dLG[g]=grid.extrapolateData(self.getMaxSpace())

            K=np.zeros((g_c.size,len(dLG)),dtype=complex)

            local_count=0
            for g in self.getRelevantGridSizes():
                K[:,local_count] = dLG[g].flatten()
                K[:,local_count] -= l_c * dG[g].flatten()
                K[:,local_count] -= g_c.flatten() * dL[g]

                local_count+=1

            KK = np.dot(K.conjugate().T,K)
            Krc = np.dot(K.conjugate().T,r_c.flatten())

            c = np.linalg.solve(KK,Krc)

            local_count=0
            for g in self.getRelevantGridSizes():
                d_g +=c[local_count] * dG[g]
                d_l +=c[local_count] * dL[g]
                local_count+=1


            damping = 1.0

            g_c -= d_g * damping
            l_c -= d_l * damping

            ##residual computation
            storeInCheckpoint(g_c,geneEnv_copy.diagDir,chpt_name='checkpoint')
            storeInCheckpoint(g_c,geneEnv_copy.diagDir,chpt_name='checkpoint_in')
            geneEnv_copy.parFileIO.updateFile()
            geneEnv_copy.runGENE()
            cpWithL = checkpoint2Local(geneEnv_copy.diagDir)
            providerWithL = DataProviderCP(cpWithL,useMapProxies=False)
            ref_grid = IterativeRefinementEVOpticomGridEigenvector(self.getMaxSpace(),providerWithL)

            d = ref_grid.extrapolateData(self.getMaxSpace())

            r_c = d - l_c*g_c

            if log_dict is not None:
                    log_dict['l'].append(l_c)
                    log_dict['g'].append(g_c.copy())
                    log_dict['residual'].append(r_c.copy())

            print "{:+5e}\t{:+5e}\t".format(np.linalg.norm(d),np.linalg.norm(g_c))

            self.log.debug(str(self.getMaxSpace()))
            self.log.debug(str(d.shape))
            self.log.debug(str(g_c.shape))
            self.log.debug(str(r_c.shape))
            self.log.debug(str(l_c))
            self.log.debug(str(np.linalg.norm(r_c)))
            # self.log.info(str(l_c))
            # self.log.info(str(np.linalg.norm(g_c)))

            print '======================================================================================='
            print 'coef: ',c
            print "d_g: {:.10e}\tr_c: {:.3e}\td_l: {:.8e}\tl: {:.4e}".format(np.linalg.norm(d_g),np.linalg.norm(r_c),d_l,l_c)
            # g_c/=np.linalg.norm(g_c)

        return g_c,l_c,r_c

    def refinementFullGrid(self,iterations=10,g_c_in=None,l_c_in=None,g_c_grid=None,a_tol='1e-7',r_tol='1e-10'):
        """
        Gene specific implementation
        works, but takes a lot of time
        """
        ##create reference grid
        geneEnv = self.gridsDict[list(self.getRelevantGridSizes())[0]].iterativeOperatorGeneEnv
        geneEnv_copy = geneEnvironment2( \
            copy(geneEnv.genePath), \
            copy(geneEnv.problemFolder),
            geneEnv.genePath+geneEnv.problemFolder+'output/placeholder_for_residual/')
        geneEnv_copy.parFileIO.updateParameter('read_checkpoint','T')
        geneEnv_copy.parFileIO.updateParameter('which_ev',"'rhs_only'")
        geneEnv_copy.runCommand=copy(geneEnv.runCommand)

        l_c = self.lamdaCombined                            if l_c_in is None else l_c_in
        g_c = self.getResultFromUpdatedScheme(reshape=True) if g_c_in is None else g_c_in

        g_c_norm = np.linalg.norm(g_c)
        g_c /= g_c_norm

        storeInCheckpoint(g_c,geneEnv_copy.diagDir,chpt_name='checkpoint')
        geneEnv_copy.setGridSizeWithLevel(self.getMaxSpace())
        geneEnv_copy.parFileIO.updateFile()
        geneEnv_copy.runGENE()
        cpWithL = checkpoint2Local(geneEnv_copy.diagDir)
        providerWithL = DataProviderCP(cpWithL,useMapProxies=False)
        ref_grid = IterativeRefinementEVOpticomGridEigenvector(self.getMaxSpace(),providerWithL)
        d = ref_grid.extrapolateData(self.getMaxSpace())

        #TODO das sollte eigentlich auch mit DataProviderGene funktionieren...\
        # macht es aber nicht, da scheint wohl irgendwo ein Bug zu sein.

        r_c = d - l_c*g_c

        self.log.debug("g_c.shape:  {}\tr_c.shape: {}".format(g_c.shape,r_c.shape))

        print 'norm r_c\t',np.linalg.norm(r_c)
        print "{:+5e}\t{:+5e}\t".format(np.linalg.norm(d),np.linalg.norm(g_c))


        for i in range(iterations):
            hash_tag = str(np.linalg.norm(r_c))
            self.log.debug('Start of refinement step {}'.format(i))
            d_g = np.zeros(g_c.shape,dtype=complex)
            d_l = 0.0

            coef=1.0

            print '--------------------'

            for g in [self.getMaxSpace()]:
                ## sample r_c and g_c

                g_c_sample = g_c_grid.injection(g_c)
                r_c_sample = g_c_grid.injection(r_c)

                self.log.debug("{}\t{}".format(g_c_sample.shape,r_c_sample.shape))

                ## solve the 2 systems for Schur complement
                # A1rc = g_c_grid.solveSystem(r_c_sample*np.linalg.norm(g_c_sample),
                #                                      l_c,hash_tag,tag='r_c',a_tol=a_tol,r_tol=r_tol)
                A1rc = g_c_grid.solveSystem(r_c_sample,
                                                     l_c,hash_tag,tag='r_c',a_tol=a_tol,r_tol=r_tol)
                # A1rc = self.gridsDict[g].solveSystem(r_c_sample/np.linalg.norm(g_c_sample),
                #                                      l_c,hash_tag,tag='r_c')
                A1gc = g_c_grid.solveSystem(g_c_sample,l_c,hash_tag,tag='g_c',a_tol=a_tol,r_tol=r_tol)

                ## solve using Schur-Complement
                f2 = 0.0
                gs = g_c_sample.flatten()
                dl = (f2 + np.vdot(gs,A1rc))/ (-np.vdot(gs,A1gc))
                dg = (A1rc + A1gc*dl).reshape(g_c_sample.shape)

                ## prolongation
                self.log.debug(np.linalg.norm(dg))
                dg = g_c_grid.extrapolateData(self.getMaxSpace(),data_override=dg,no_proxy=True)
                self.log.debug(np.linalg.norm(dg))
                print "{}\t{:5e}\t{:5e}\t{:5e}\t{:5e}".format(g[0],np.linalg.norm(dg),dl,np.linalg.norm(A1gc),np.linalg.norm(g_c_sample*np.linalg.norm(g_c_sample)))

                coef = 1.0

                d_g += dg * coef
                d_l += dl * coef

                # coef = 0.0

            g_c -= d_g
            l_c -= d_l

            ##residual computation
            storeInCheckpoint(g_c,geneEnv_copy.diagDir,chpt_name='checkpoint')
            storeInCheckpoint(g_c,geneEnv_copy.diagDir,chpt_name='checkpoint_in')
            geneEnv_copy.parFileIO.updateFile()
            geneEnv_copy.runGENE()
            cpWithL = checkpoint2Local(geneEnv_copy.diagDir)
            providerWithL = DataProviderCP(cpWithL,useMapProxies=False)
            ref_grid = IterativeRefinementEVOpticomGridEigenvector(self.getMaxSpace(),providerWithL)

            d = ref_grid.extrapolateData(self.getMaxSpace())

            r_c = d - l_c*g_c

            print "{:+5e}\t{:+5e}\t".format(np.linalg.norm(d),np.linalg.norm(g_c))

            self.log.debug(str(self.getMaxSpace()))
            self.log.debug(str(d.shape))
            self.log.debug(str(g_c.shape))
            self.log.debug(str(r_c.shape))
            self.log.debug(str(l_c))
            self.log.debug(str(np.linalg.norm(r_c)))
            # self.log.info(str(l_c))
            # self.log.info(str(np.linalg.norm(g_c)))

            print '======================================================================================='
            print "d_g: {:.10e}\tr_c: {:.3e}\td_l: {:.8e}\tl: {:.4e}".format(np.linalg.norm(d_g),np.linalg.norm(r_c),d_l,l_c)
            # g_c/=np.linalg.norm(g_c)

        return g_c,l_c,r_c

    def refinementOld(self,iterations=10):
        ##get lambda_c and g_c
        l_c = self.lamdaCombined
        g_c = self.getResultFromUpdatedScheme(reshape=True)
        r_c = self.getResidualVector().reshape(g_c.shape)
        self.log.debug(str(g_c.shape)+'  '+str(r_c.shape))
        hash_tag = str(np.linalg.norm(r_c))

        d_g = None
        d_l = 0.0

        for i in range(iterations):
            self.log.debug('Start iterative refinement step '+str(i))
            ## project on subrids
            for g in self.getRelevantGridSizes():
                '''create a combined correction of the initial solution'''
                assert isinstance(self.gridsDict[g],IterativeRefinementEVOpticomGridEigenvector)
                self.log.debug(g[0])
                M1rc = self.gridsDict[g].solveSystem(r_c,l_c,hash_tag,tag='r_c')
                M1gc = self.gridsDict[g].solveSystem(g_c,l_c,hash_tag,tag='g_c')
                coef = self.getSchemeDict()[g[0]]
                self.log.debug(str(M1rc.shape))
                self.log.debug(M1gc.shape)
                self.log.debug(self.s.shape)
                self.log.debug(np.linalg.norm(M1rc))
                self.log.debug(np.linalg.norm(M1gc))

                s = g_c

                dl = - np.vdot(s, M1rc) / np.vdot(s, M1gc)
                self.log.info(dl)

                d_l += dl * coef

                self.log.debug(g_c.shape)

                dg = -M1rc-M1gc*dl
                if d_g is None:
                    d_g =  dg * coef
                else:
                    d_g += dg * coef

            self.log.info(d_l)
            self.log.info(np.linalg.norm(dg))
            g_c = g_c + d_g
            l_c = l_c + d_l

            ##solve large system for updated residual
            ##create reference grid
            geneEnv = self.gridsDict[list(self.getRelevantGridSizes())[0]].iterativeOperatorGeneEnv
            geneEnv_copy = geneEnvironment2(\
                copy(geneEnv.genePath),\
                copy(geneEnv.problemFolder),
                geneEnv.genePath+geneEnv.problemFolder+'output/placeholder_for_residual/')
            geneEnv_copy.parFileIO.updateParameter('read_checkpoint','T')
            geneEnv_copy.parFileIO.updateParameter('which_ev',"'rhs_only'")
            # geneEnv_copy.parFileIO.updateParameter('ev_shift_ref', '('+str(l_c.real)+','+str(l_c.imag)+')')
            # geneEnv_copy.setGridSizeWithLevel(self.getMaxSpace())


            storeInCheckpoint(g_c,geneEnv_copy.diagDir,chpt_name='checkpoint')


            ref_provider = DataProviderGene(geneEnv_copy)
            ref_provider.setLevel(self.getMaxSpace())
            ref_grid = IterativeRefinementEVOpticomGridEigenvector(self.getMaxSpace(),ref_provider)
            ref_grid.fillData()
            d= ref_grid.extrapolateData(self.getMaxSpace())
            self.log.debug(str(self.getMaxSpace()))
            self.log.debug(str(d.shape))
            self.log.debug(str(g_c.shape))
            self.log.debug(str(r_c.shape))
            self.log.debug(str(l_c))
            r_c = d - l_c*g_c
            self.log.debug(str(np.linalg.norm(r_c)))
            self.log.info(str(l_c))
            self.log.info(str(np.linalg.norm(g_c)))

        return r_c
