'''
Created on Nov 23, 2013

@author: kowitz
'''
import copy as cp
from numpy.core.umath import pi
#from numpy.lib._compiled_base import unravel_index
from numpy.lib import unravel_index
from os import mkdir
from combiGrid import combiGrid
import abc
from GeneInterface import checkpoint2Local
import numpy
import logging
from scipy.ndimage import map_coordinates
from numpy.lib.index_tricks import mgrid
from numpy.core.numeric import concatenate, zeros
from numpy import array, ceil,copy
import sys, inspect
from numpy.ma.extras import unique, dot
import GeneInterface
from multiprocessing import Pool, cpu_count
from numpy.ma.core import log2, around, log, cos
from numpy.lib.npyio import save, load
import os
from argparse import ArgumentError
from functools import wraps
import struct
from array import array as ar
from symbol import except_clause
import shutil
from scipy.ndimage.interpolation import zoom
import numpy as np


class DataProvider(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def fillData(self, *args):
        return

    def getExtrapolationFromProxy(self, *args, **kwargs):
        """
        Dummy for storing the extrapolated results. Can be overridden if proxies should be saved. Otherwise it should
        return None
        """
        return None

    def storeExtrapolationProxy(self, *args, **kwargs):
        """
        Dummy for storing the proxy
        """
        pass


class DummyProviderIV(DataProvider):
    def __init__(self, nSpecies=1):
        self.log = logging.getLogger(__name__)
        self.log.debug('Created dummy provider')
        self.nSpecies = nSpecies


    def fillData(self, level):
        self.log.debug('fillData of Dummy')
        l = [self.nSpecies]
        l.extend(list(2 ** array(level)))

        l[5] += 1
        l[4] -= 1
        l[3] += 1
        l[2] -= 1
        l[1] += 1

        self.data = [zeros(l, dtype=complex)]
        for i in range(l[3]):
            self.data[0][:, :, :, i, :, :] = i

        for i in range(l[5]):
            self.data[0][:, :, :, :, :, i] += 1.0j * (i - l[5] / 2)


class HierarchicalBasisProviderGene(DataProvider):

    def __init__(self, basis_level, index, root_dir, n_species=1,spec_index=0):
        """
        This takes the root dir of the outputs and extrapolated outputs and the restarts. Its implemented with
        boundaries. If a boundary basis is considered should probably be steered by the used index. The number of
        species determines the size of the first dim of the actual array

        :param n_species: number of species
        :param index: flattened index of the basis function
        :param basis_level: Index of the hierarchical basis function
        :param root_dir: directory, in which the data has to be stored
        :type root_dir: str
        """
        self.spec_index = spec_index
        self.n_species = n_species
        self.index = index
        self.root_dir = root_dir
        self.log = logging.getLogger(__name__)
        self.level = basis_level

        #create folder for this basisfunction
        self.folder_name = self.root_dir+'hiera_'+str(basis_level)+'_'+str(index)+'_spec_'+str(self.spec_index)+'/'

        if not os.path.exists(self.folder_name):
            os.makedirs(self.folder_name)

        self.log.debug('Created hierarchical Grid provider')

    def fillData(self):
        """
        Create minimally resolved hat basis according to level string
        """

        dim = len(self.level)
        grid_sizes = zeros(dim + 1)
        grid_sizes[0] = self.n_species
        grid_sizes[1:] = 2 ** array(self.level) + 1
        grid_sizes = tuple(grid_sizes)
        basis = zeros(grid_sizes, dtype=complex)
        # print basis.shape,grid_sizes
        #Berechnung des Indexes
        f = lambda n: 3 if n==1 else 2**(n-1)
        start = lambda n: 0 if n==1 else 1
        stride = lambda n: 1 if n==1 else 2
        startx = lambda n: 0 if n==1 else 2**n+1 - 2**(n-1)

        inner_points = map(f,self.level)

        ind = unravel_index(self.index,inner_points)
        self.log.debug(str(list(ind)))
        ind = tuple(list(ind))
        l = self.level
        # print self.index,inner_points,map(start,self.level),map(stride,self.level),ind
        basis[self.spec_index,\
                start(l[0])::stride(l[0]),\
                start(l[1])::stride(l[1]),\
                start(l[2])::stride(l[2]),\
                start(l[3])::stride(l[3]),\
                startx(l[4]):][ind] = 1.0

        #reorder for x direction
        def computeIndices(n):
            offset = n/2
            l = []
            for i in range(n):
                if i%2==0:
                    l.append(i/2 + offset)
                else:
                    l.append(-(i+1)/2 + offset)
            return l

        new_x_index = computeIndices(2**self.level[4]+1)
        #print basis[:,:,:,:,:].shape,new_x_index

        #basis = basis[:,:,:,:,:,[new_x_index]]
        new_basis = basis.copy()
        for i in range(2**l[4]+1):
            basis[:,:,:,:,:,new_x_index[i]] = new_basis[:,:,:,:,:,i]


        self.data = [basis]

    def getDir(self):
        return self.folder_name

    def getExtrapolationProxyFilename(self, l, flag):
        return 'basis_' + str(self.index) + '_' + '_on_' + str(tuple(around(l).astype(int))) + '_' + flag

    def getExtrapolationFromProxy(self, l, flag='default'):
        dirList = os.listdir(self.getDir())
        fileName = self.getExtrapolationProxyFilename(l,flag) + '.npy'
        self.log.debug(fileName)
        if fileName in dirList:
            ret = load(self.getDir() + fileName)
            if ret.shape[4]==1:
                    ret=np.concatenate((np.zeros(ret.shape),ret,np.zeros(ret.shape)),axis=4)
            return ret
        else:
            self.log.debug('File not found in dir')
            return None

            #     def setGridDict(self, inpardict):
            #         self.gridInParFileDict = inpardict
            #

    def storeExtrapolationProxy(self, data, flag='default'):
        #FIXME this is probably not working for fields
        s = array(data.shape)[1:]

        l = log2(s - 1)
        fileName = self.getDir() + self.getExtrapolationProxyFilename(l, flag)
        #         fileHandle = open(fileName,'wb')
        ##store only on y coordinate!
        if data.shape[4]==3:
                save(fileName,data[:,:,:,:,1,:])
        else:
                save(fileName, data)
        self.log.info('Stored extrapolated data in ' + fileName)

class HieraProviderGene(DataProvider):

    hat_proxy= None

    def __init__(self,basis_level, index, root_dir, n_species=1,spec_index=0,boundaries=(True,True,True,False,True)):
        """
        This takes the root dir of the outputs and extrapolated outputs and the restarts. Its implemented with
        boundaries. If a boundary basis is considered should probably be steered by the used index. The number of
        species determines the size of the first dim of the actual array

        :param n_species: number of species
        :param index: flattened index of the basis function
        :param basis_level: Index of the hierarchical basis function
        :param root_dir: directory, in which the data has to be stored
        :type root_dir: str
        """
        self.boundaries = boundaries
        self.spec_index = spec_index
        self.n_species = n_species
        self.index = index
        self.root_dir = root_dir
        self.log = logging.getLogger(__name__)
        self.level = basis_level

        #create folder for this basisfunction
        self.folder_name = self.root_dir+'hiera_'+str(self.level)+'/'+str(index)+'_spec_'+str(self.spec_index)+'/'

        if not os.path.exists(self.folder_name):
            os.makedirs(self.folder_name)

        self.log.debug('Created hierarchical Grid provider')

    def getDir(self):
        return self.folder_name

    def getSubspaceGrid(self):
        f = lambda n,b: 1+2*b if n==1 else 2**(n-1)
        b=self.boundaries
        subspace_grid = map(f,self.level,b)

        #for i in range(len(subspace_grid)):
        #    if not self.boundaries[i]:
        #        subspace_grid[i]-=2

        return subspace_grid

    def get_udated_index(self):
        ind = self.get_index()
        grid_sizes=self.get_grid_sizes()
        new_ind = lambda n,b: n if b else 1+2*n
        #new_ind_fourier = lambda i,n,b:n/2+i/2+(1-1*b) if i%2==0 else n/2-(i+1)/2+(1-1*b)
        #new_ind_fourier = lambda i,n,b: i/2 -n/4 if i%2==0 else n/4-(i+1)/2
        def new_ind_fourier(i,n,b):
            if n==3:
                if b:
                    l=[1,0,2]
                    return l[i]
                else:
                    assert i==0
                    return 1
            else:
                if b:
                    return i/2 -n/4 if i%2==0 else n/4-(i+1)/2
                else:
                    return i/2 -n/4-1 if i%2==0 else n/4-(i+1)/2+1


        subspace_grid=self.getSubspaceGrid()
        b = self.boundaries
        l=self.level
        ind_updated =(\
            new_ind(ind[0],b[0]),\
            new_ind(ind[1],b[1]),\
            new_ind(ind[2],b[2]),\
            new_ind_fourier(ind[3],grid_sizes[-2],b[3]),\
            new_ind_fourier(ind[4],grid_sizes[-1],b[4]))
        #print ind_updated,subspace_grid[-1], grid_sizes
        return ind_updated

    def get_index(self):
        subspace_grid = self.getSubspaceGrid()
        #print subspace_grid
        ind = unravel_index(self.index,subspace_grid)
        #print 'ind ',ind
        return ind

    def compute_classifier(self,ind_updated):
        grid_sizes=self.get_grid_sizes()
        classifier=[1]*3

        for i in range(3):
            b= self.boundaries[i]
            if b:
                if ind_updated[i]==0:
                    classifier[i]=0
                elif ind_updated[i]==grid_sizes[i+1]-1:
                    classifier[i]=2
        self.classifier=tuple(classifier)

    def get_grid_sizes(self):
        dim = len(self.level)
        grid_sizes = zeros(dim + 1,dtype=int)
        grid_sizes[0] = self.n_species
        grid_sizes[1:] = 2 ** array(self.level) + 1
        grid_sizes = tuple(grid_sizes)
        return grid_sizes

    def fillData(self):
        ## get large array
        grid_sizes=self.get_grid_sizes()
        basis = zeros(grid_sizes, dtype=complex)

        ## get Index in subspace
        #ind = self.get_index()
        ind_updated = self.get_udated_index()

        ## classify which kind of boundary point, only for space coord
        self.compute_classifier(ind_updated)
        #print self.classifier

        #cube= self.unit_cube()

       # cube[self.classifier]=1.0

        ##das kann man mit proxies abkuerzen
        ##hat = zoom(cube,[grid_sizes[1],grid_sizes[2],grid_sizes[3]],order=1)

        basis[self.spec_index][ind_updated]=1.0
        self.index_hat = ind_updated
        self.data=[basis]

    def unit_cube(self,dim=3):
        shape = tuple([3]*dim)
        return zeros(shape)

    def getExtrapolationFromProxy(self, level,  *args,**kwargs):
        level_out= level
        self.fillData()


        cube = self.unit_cube()
        self.log.debug(self.classifier)
        cube[self.classifier]=1.0

        # ranges=[]

        #
        # cut_cube=cube[ranges[0][0]:ranges[0][1]+1,ranges[1][0]:ranges[1][1]+1,ranges[2][0]:ranges[2][1]+1]
        # cube_size= cut_cube.shape

        new_size=2**array(level_out,dtype=numpy.float64)+1
        old_size=2**array(self.level,dtype=numpy.float64)+1
        cube_size=2**array([1,1,1],dtype=float)+1

        l_diff = array(level_out)-array(self.level)
        self.log.debug(str(l_diff))
        hat_size=2**l_diff + 1
        hat_size[hat_size==2]=3
        ## TODO das kann man noch ueber proxy machen
        self.log.debug(str(cube.shape)+ ' '+str(new_size[0:3]/cube_size[0:3]))
        self.log.debug(str(new_size)+' '+str(old_size)+ ' ')
        self.log.debug(str(hat_size[0:3]/cube_size[0:3]))
        hat = zoom(cube,hat_size[0:3]/cube_size[0:3],order=1)

        self.log.debug(str(self.classifier) + ' ' + str(hat.shape))

        support = array(list((array(hat.shape)-1)/2)+[1,1])


        if self.classifier[0]==0:
            hat=hat[0:hat.shape[0]/2+1,:,:]
        elif self.classifier[0]==2:
            hat=hat[hat.shape[0]/2:,:,:]

        if self.classifier[1]==0:
            hat=hat[:,0:hat.shape[1]/2+1,:]
        elif self.classifier[1]==2:
            hat=hat[:,hat.shape[1]/2:,:]

        if self.classifier[2]==0:
            hat=hat[:,:,0:hat.shape[2]/2+1]
        elif self.classifier[2]==2:
            hat=hat[:,:,hat.shape[2]/2:]



        index= self.get_udated_index()
        ##transfer index into the large grid
        updated_index = tuple(array(index)*2**(l_diff))
        self.log.debug(str(updated_index))

        #support = (old_size-1)/2

        self.log.debug(str(support))

        lower=array(updated_index)-support
        upper=array(updated_index)+support
        self.log.debug('lower and upper: '+str(lower) + '  ' + str(upper))
        lower[lower<0] = 0
        upper[upper>new_size] = new_size[upper>new_size]
        self.log.debug('lower and upper: '+str(lower) + '  ' + str(upper))
        grid_sizes=self.get_grid_sizes()
        basis = zeros([self.n_species]+list(new_size), dtype=complex)
        #print 'bla', basis[self.spec_index,lower[0]:upper[0]+1,lower[1]:upper[1]+1,lower[2]:upper[2]+1,updated_index[-2],updated_index[-1]].shape
        self.log.debug(str(hat.shape))
        basis[self.spec_index,lower[0]:upper[0]+1,lower[1]:upper[1]+1,lower[2]:upper[2]+1,updated_index[-2],updated_index[-1]]=hat


        ## compute borders of array to set according to
        subspace_index = self.get_index()

        return basis




class CheckpointProvider(DataProvider):
    def __init__(self, checkpoint, useMapProxies=False):
        self.cp = checkpoint
        self.useProxies(useMapProxies)

    def useProxies(self, val):
        if val:
            self.storeExtrapolationProxy = self.storeProxy
        else:
            self.storeExtrapolationProxy = self.empty

    def empty(self, *args, **kwargs):
        pass

    def getDir(self):
        return self.cp.diagDir

    def get_mu_grid_type(self):
        return self.cp.getPar('mu_grid_type')

    def storeExtrapolationProxy(self, *args, **kwargs):
        self.log.debug('proxies can be switched on')
        pass

    def getExtrapolationFromProxy(self, l, flag='default'):
        dirList = os.listdir(self.getDir())
        fileName = str(self.cp.getLevel()) + '_on_' + str(tuple(l)) + '_' + flag + '.npy'
        self.log.debug(fileName)
        if fileName in dirList:
            ret  = load(self.cp.diagDir + fileName)
        #    print ret.shape
        #     if ret.shape[4]==1:
            if ret.shape[-2]==1:
                    ret = np.concatenate(
                                    (np.zeros(ret.shape,dtype=complex),ret,np.zeros(ret.shape,dtype=complex))
                                    ,axis=4
                                    )
        #    print ret.shape
            return ret
        else:
            self.log.debug('File not found in dir')
            return None

            #     def setGridDict(self, inpardict):
            #         self.gridInParFileDict = inpardict
            #

    def storeProxy(self, data, flag='default'):
        #FIXME this is not working for fields!
        s = array(data.shape)[1:]
        assert s.size==5
        l = log2(s - 1)
        fileName = self.getDir() + str(self.cp.getLevel()) + '_on_' + str(tuple(around(l).astype(int))) + '_' + flag
        #         fileHandle = open(fileName,'wb')
        #print 'store',data.shape
        # if data.shape[4]==3:
        if data.shape[-2]==3:
                save(fileName,data[...,[1],:])
        else: 
                save(fileName, data)
        self.log.info('Stored extrapolated data in ' + fileName)

    def fillDataFromEV(self):
        self.log.debug('fill data from EV checkpoint')
        self.data = self.cp.eigenvectors  # [:][:, :, 1:, :, :, :]
        # # remove point in v
        #         for i in range(len(self.data)):
        #             self.data[i] = self.data[i][:, :, 1:, :, :, :]
        self.EV = self.cp.EV

        # # add 0 point for mu,v
        s = numpy.array(self.data[0].shape)
        self.log.debug(s)
        s[1] = 1
        for i in range(len(self.data)):
            self.data[i] = numpy.append(self.data[i], numpy.zeros(s), 1)

        s = numpy.array(self.data[0].shape)
        s[2] = 1
        for i in range(len(self.data)):
            self.data[i] = numpy.append(self.data[i], numpy.zeros(s), 2)

        self.log.info('created EV combigrid of size ' + str(self.data[0].shape))

    def fillDataFromIV(self):
        self.data = self.cp.snapshots  # missingGrids.extend([lmax])# [:][:, :, 1:, :, :, :]
        # # remove point in v
        #         for i in range(len(self.data)):
        #             self.data[i] = self.data[i][:, :, 1:, :, :, :]

        # # add 0 point for mu,v
        s = numpy.array(self.data[0].shape)
        s[1] = 1
        for i in range(len(self.data)):
            self.data[i] = numpy.append(self.data[i], numpy.zeros(s), 1)

        s = numpy.array(self.data[0].shape)
        s[2] = 1
        for i in range(len(self.data)):
            self.data[i] = numpy.append(self.data[i], numpy.zeros(s), 2)
        self.log.info('created IV combigrid of size ' + str(self.data[0].shape))

    def getEV(self,nEV=0):
        return self.cp.EV[nEV]


class DataProviderGene(CheckpointProvider):
    def __init__(self, geneEnv):
        self.log = logging.getLogger(__name__)
        self.geneEnv = geneEnv
        # self.createParameterFile()
        self.log.debug('Created ' + self.__class__.__name__)
        self.gridInParFileDict = None

    def setGridInParFileDict(self, gridInParFileDict):
        self.gridInParFileDict = gridInParFileDict
        raise NotImplementedError('please revise')

    def getDir(self):
        return cp.copy(self.geneEnv.diagDir)

    def setLevel(self, level):

        self.gridInParFileDict = {'nx0': str(2 ** level[4] + 1), 'nky0': str(2 ** level[3] - 1), \
                                  'nz0': str(2 ** level[2]), 'nv0': str(2 ** level[1]), 'nw0': str(2 ** level[0])}

    def getExtrapolationFromProxy(self, l, flag='default'):
        return None


    def createParameterFile(self):
        '''
        Probably best to do that just before running GENE
        '''
        gridDict = self.gridInParFileDict

        # ## adjust grid sizes to even numbers
        for k in gridDict.iterkeys():
            self.geneEnv.parFileIO.updateParameter(k, str(gridDict[k]))
            self.log.debug('Adjusted ' + k + ' to ' + str(gridDict[k]))

        # ## adjust boundaries of grids
        # # z -- automatically
        # # v
        # TODO hier gehts weiter...anpassung des gitters
        # # check correct source of GENE
        # # v kann man erstmal konstant lassen --> spaeter anpassen, Momment ist halt ein punkt auf der 0

        # # mu?
        # # muss angepasst werden
        # # mu muss regular grid sein!
        # # ist angepasst

        #self.geneEnv.parFileIO.updateParameter('mu_grid_type', "'equidist'")
        if self.geneEnv.parFileIO.getPar('comp_type') == 'EV':
            self.log.debug('is EV geneEnv')
            self.isEV = True
            self.nEV = 0
        else:
            self.log.debug('is IV geneEnv')
            self.isEV = False
            self.snapshotNo = -1

    def fillData(self):
        if self.gridInParFileDict is None:
            raise ValueError('grid sizes have not been set yet')
        self.createParameterFile()
        self.geneEnv.runGENE()
        self.cp = checkpoint2Local(self.geneEnv.diagDir)
        self.cp.createAllData()

        if self.isEV == True:
            self.fillDataFromEV()
        else:
            self.fillDataFromIV()

class DataProviderGeneRestart(DataProviderGene):

    def __init__(self,geneEnv,chpt_in_name='checkpoint_extra',*args,**kwargs):
        DataProviderGene.__init__(self,geneEnv,*args,**kwargs)
        self.chpt_in_name = chpt_in_name


    def fillData(self):
        if self.gridInParFileDict is None:
            raise ValueError('grid sizes have not been set yet')
        self.createParameterFile()

        shutil.copy(self.geneEnv.diagDir+self.chpt_in_name, self.geneEnv.diagDir+'checkpoint')

        self.geneEnv.runGENE()
        self.cp = checkpoint2Local(self.geneEnv.diagDir)
        self.cp.createAllData()

        if self.isEV == True:
            self.fillDataFromEV()
        else:
            self.fillDataFromIV()


class DataProviderCP(CheckpointProvider):
    def __init__(self, cp, useMapProxies=True):
        self.useProxies(useMapProxies)
        self.cp = cp

        self.log = logging.getLogger(__name__)
        if self.cp.getPar('comp_type') == 'EV':
            self.log.debug('is EV geneEnv')
            self.isEV = True
            self.nEV = 0
        # #check size of grid
        else:
            self.log.debug('is IV geneEnv')
            self.isEV = False
            self.snapshotNo = -1

        self.gridInParFileDict = None
        self.log.debug('Created ' + self.__class__.__name__)

    def fillData(self,smoothing=False):
        #         if self.gridInParFileDict == None:
        #             raise ValueError('grid sizes have not been set yet')
        #         d = self.gridInParFileDict
        #         assert self.cp.res[:-1] == (d['nx0'], d['nky0'], d['nz0'], d['nv0'], d['nw0'])
        self.cp.createAllData()

        if self.isEV:
            self.fillDataFromEV()
        else:
            self.fillDataFromIV()


class geneCombinationGrid(combiGrid):
    '''
    alternate implementation of the combigrid using a provider and viewer
    '''


    def __init__(self, levelvector, checkpointProvider, \
                 order=1, boundaries=(True, True, True, False, True),normalization=None):
        '''
        constructor for a local version of the code
        '''
        combiGrid.__init__(self, levelvector, boundaries)



        self.provider = checkpointProvider

        self.order = order

        self.coordinateNames = ['nx0', 'nky0', 'nz0', 'nv0', 'nw0'][::-1]
        self.log.debug(self.coordinateNames)
        if len(levelvector) != len(self.coordinateNames):
            raise IndexError('Levelvector not of appropriate size for GENE')
        if len(boundaries) != len(self.coordinateNames):
            raise IndexError('boundaries not of appropriate size for GENE')

        self.gridInParFileDict = {}
        self.boundaryDict = {}
        for i in range(len(self.coordinateNames)):
            self.gridInParFileDict[self.coordinateNames[i]] = self.gridSizes[i]
            self.boundaryDict[self.coordinateNames[i]] = self.boundaries[i]

        # # we need the boundaries to construct the right grid with GENE
        if self.boundaryDict['nz0'] == False:
            raise IndexError('boundary of z has to be true!')
        if self.boundaryDict['nv0'] == False:
            raise IndexError('boundary of v has to be true!')
        if self.boundaryDict['nw0'] == False:
            raise IndexError('boundary of w has to be true!')

        self.adjustGridSizes()

        self.log.debug('grid Dict: ' + str(self.gridInParFileDict))
        self.log.debug('Boundary Dict: ' + str(self.boundaryDict))

        self.log.info('Constructed geneCombiGrid of size' + str(self.levelvec))

        self.normalization=normalization

    def adjustGridSizes(self):
        # #z direction
        self.gridInParFileDict['nz0'] -= 1
        # # v direction
        # TODO shouldn't that be a minus here?
        self.gridInParFileDict['nv0'] += 1
        # # w direction
        self.gridInParFileDict['nw0'] -= 1

        self.log.debug('adjusted grid sizes for parameterfile')

    def fillData(self, *args):
        self.provider.fillData(*args)  # missingGrids.extend([lmax])

    def storeCheckpointForRestart(self,level,chpt_name='checkpoint_extra',tag=None):
        if tag is None:
            pathToCheckpoint = self.provider.getDir() + 'restart_' + str(level) + '/'
        else:
            pathToCheckpoint = self.provider.getDir() + 'restart_' + str(level) + '_'+tag+'/'
        self.log.debug(pathToCheckpoint)
        if not os.path.exists(pathToCheckpoint):
            os.makedirs(pathToCheckpoint)
	shutil.copy(self.provider.getDir()+'checkpoint',pathToCheckpoint+chpt_name);


    def storeExtrapolatedDataForRestart(self, level,chpt_name= 'checkpoint_extra',tag=None):
        if isinstance(self,geneCombinationGridFieldEigenvector):
            buf = geneCombinationGridEigenvector(self.levelvec,self.provider,nEV = self.nEV)
        else:
            buf = self

        extrapolated_data = buf.extrapolateData(level)
        # pathToCheckpoint = self.provider.cp.diagDir + 'restart_' + str(level) + '/'
        if tag is None:
            pathToCheckpoint = self.provider.getDir() + 'restart_' + str(level) + '/'
        else:
            pathToCheckpoint = self.provider.getDir() + 'restart_' + str(level) + '_'+tag+'/'
        self.log.debug(pathToCheckpoint)
        GeneInterface.storeInCheckpoint(extrapolated_data,pathToCheckpoint=pathToCheckpoint,chpt_name=chpt_name)


    def storeDataInAppropriateSize(self,extrapolatedData,tag=None):
        """
        Stores the data ( can be large) in a separate dir with the appropriate size. This should be removed after \
        usage or get an unique identifier, which is the 2 norm of the incoming data
        """
        #TODO full weighted injection has to be implemented here!

        ##downsample
        # hash_identifier = str(np.linalg.norm(extrapolatedData))
        old_size = array(extrapolatedData.shape,dtype=int)
        self.log.debug(str(old_size))
        assert (old_size.shape[0])==6
        old_size-=array([0,1,1,1,1,1])
        new_size = array([old_size[0]]+list(2**array(self.levelvec)))

        old_level = (np.rint(log2(old_size[1:])))
        new_level = (np.rint(log2(new_size[1:])))

        self.log.debug(str(old_level)+'   '+str(new_level))
        self.log.debug(str(old_size)+'   '+str(new_size))

        diff_level = old_level-new_level
        strides = diff_level +1
        self.log.debug(strides)
        self.log.debug(diff_level)
        assert (diff_level>=0).all()

        buf=extrapolatedData[:,
            ::strides[0],
            ::strides[1],
            ::strides[2],
            :,:]

        # if strides[3]>1:
        #     # self.log.debug(str(2**strides[3]-1)+'  '+str(-(2**strides[3]-1)-1)+'  '+str(buf.shape[4]))
        #     buf = buf[:,:,:,:,2**strides[3]-1:-(2**strides[3]-1),:]
        # if strides[4]>1:
        #     # self.log.debug(str(2**strides[4]-1)+'  '+str(-(2**strides[4]-1)-1)+'  '+str(buf.shape[5]))
        #     buf = buf[:,:,:,:,:,2**strides[4]-1:-(2**strides[4])-1]

        self.log.debug(str(diff_level))

        if diff_level[3]>0:
            start_ind = 2**diff_level[3]/2
            end_ind = -start_ind
            self.log.debug(start_ind,end_ind)
            buf = buf[:,:,:,:,start_ind:end_ind,:]

        if diff_level[4]>0:
            start_ind = 2**diff_level[4]/2
            end_ind = -start_ind
            buf = buf[:,:,:,:,:,start_ind:end_ind]

        self.log.debug(str(buf.shape))
        self.log.debug('created new downsampled array with strides '+str(strides))


        ##path to new checkpoint -- temporary
        pathToCheckpoint = self.provider.getDir() + 'iter_ref_'#+hash_identifier
        if tag is not None:
            pathToCheckpoint = pathToCheckpoint+tag

        pathToCheckpoint = pathToCheckpoint+'/'

        self.log.debug('Store data in folder '+pathToCheckpoint)
        self.log.debug(str(buf.shape))
        GeneInterface.storeInCheckpoint(buf,pathToCheckpoint,chpt_name='checkpoint_iter_ref')





def mapCoordWrapper(inputTuple, order=1):
    return map_coordinates(inputTuple[0], inputTuple[1], order=order)

class geneCombinationGridField(geneCombinationGrid):
    """ Grid for combining the field instead of the g
    """

    def __init__(self, levelvector, checkpointProvider, order=1, boundaries=(True, True, True, False, True)):
        super(geneCombinationGridField, self).__init__(levelvector, checkpointProvider, order, boundaries)
        self.log.debug('Created a combination grid for combining fields')
        self.changeStrategy('default')

    def extrapolateData(self,level,*args,**kwargs):
        raise NotImplementedError('Please do not call')

    def changeStrategy(self, name):
        if name == 'regular' or name == 'default':
            self.extrapolateData = self.extrapolateDataRegular
            self.strategy = 'default'
        elif name == 'abs':
            self.extrapolateData = self.extrapolateDataAbs
            self.strategy = 'abs'
        else:
            raise ArgumentError('wrong strategy chosen')

    def getExtrapolatedDataSize(self, l):
        '''Might not work, if the cp has not been created yet'''
        return list([2 ** l[2] + 1, 2 ** l[3] + 1,
            2 ** l[4] + 1])

    def getPadding(self):
        buf = self.getData()
        s = 2 ** array(self.levelvec) + 1
        s = list(s)[2:]
        res = zeros(s, dtype=complex)
        start = array(self.boundaries) * -1 + 1

        self.log.debug(str(s))
        self.log.debug(str(start))

        # #padding
        res[start[2]:s[0] - start[2],\
        start[3]:s[1] - start[3],\
        start[4]:s[2] - start[4]] = buf
        return res

    def extrapolateDataRegular(self, level, providerIn=None):
        '''changes for global runs'''

        if providerIn == None:
            provider = self.provider
        else:
            provider = providerIn

        # #check for proxies first...eigentlich im provider
        if hasattr(self,'nEV'):
            flag = 'default_ev_' + str(self.nEV)
        elif hasattr(self,'snapshot'):
            flag = 'default_snapshot_{}'.format(self.snapshot)
        else:
            raise AttributeError('runtype unknown')
        flag += '_field'
        flag += '_ord_' + str(self.order)

        buf = provider.getExtrapolationFromProxy(level, flag=flag)

        if buf != None:
            self.log.info('Loaded Extrapolation from Proxy')
            return buf
        #         processPool = Pool(processes=self.nProcesses)
        self.fillData()

        buf = self.getPadding()

        s = buf.shape  # shape of the original array - should be the boundaries
        l = self.getExtrapolatedDataSize(level)
        self.log.debug(str(s))
        self.log.debug(str(l))

        ## real space coordinates z,v,w
        spaceLevels = (array(l) * 1.0) / array(s)
        spaceLevels[1] = 1.0
        spaceLevels[2] = 1.0

        self.log.info('Start zoom procedure with zoomFactor ' + str(spaceLevels))

        if not (spaceLevels == 1.0).all():
            realBuf = zoom(buf.real, spaceLevels, order=self.order)
            imagBuf = zoom(buf.imag, spaceLevels, order=self.order)
        else:
            realBuf = buf.real
            imagBuf = buf.imag

        ### add 0 y boundary
        self.log.debug('Start y Padding')
        yPadding = l[1] - s[1]
        if yPadding > 0:
            assert yPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[1] = yPadding / 2
            padding = zeros(paddingShape)
            for b in [realBuf, imagBuf]:
                b = concatenate((padding, b, padding), axis=1)
        elif yPadding < 0:
            yPadding *= -1
            assert yPadding % 2 == 0
            for b in [realBuf, imagBuf]:
                b = b[  :, yPadding / 2:-yPadding / 2, :]

        ### add 0 x boundary
        self.log.debug('start x-padding')
        self.log.debug(realBuf.shape)
        xPadding = l[2] - s[2]
        if xPadding > 0:
            assert xPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[2] = xPadding / 2
            padding = zeros(paddingShape)
            realBuf = concatenate((padding, realBuf, padding), axis=2)
            imagBuf = concatenate((padding, imagBuf, padding), axis=2)
        elif xPadding < 0:
            xPadding *= -1
            assert xPadding % 2 == 0
            realBuf = realBuf[ :, :, xPadding / 2:-xPadding / 2]
            imagBuf = imagBuf[ :, :, xPadding / 2:-xPadding / 2]

        self.log.debug(realBuf.shape)

        buf = (realBuf + 1.j * imagBuf)
        provider.storeExtrapolationProxy(buf, flag=flag)

        return buf

    def extrapolateDataAbs(self, level, providerIn=None):
        '''changes for global runs'''

        if providerIn == None:
            provider = self.provider
        else:
            provider = providerIn

        # #check for proxies first...eigentlich im provider

        flag = 'abs_ev_' + str(self.nEV)
        flag += '_field'
        flag += '_ord_' + str(self.order)

        buf = provider.getExtrapolationFromProxy(level, flag=flag)

        if buf != None:
            self.log.info('Loaded Extrapolation from Proxy')
            return buf
        #         processPool = Pool(processes=self.nProcesses)
        self.fillData()

        buf = self.getPadding()

        s = buf.shape  # shape of the original array - should be the boundaries
        l = self.getExtrapolatedDataSize(level)
        self.log.debug(str(s))
        self.log.debug(str(l))

        ## real space coordinates z,v,w
        spaceLevels = (array(l) * 1.0) / array(s)
        spaceLevels[1] = 1.0
        spaceLevels[2] = 1.0

        self.log.info('Start zoom procedure with zoomFactor ' + str(spaceLevels))

        if not (spaceLevels == 1.0).all():
            realBuf = zoom(abs(buf), spaceLevels, order=self.order)
        else:
            realBuf = abs(buf)

        ### add 0 y boundary
        self.log.debug('Start y Padding')
        yPadding = l[1] - s[1]
        if yPadding > 0:
            assert yPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[1] = yPadding / 2
            padding = zeros(paddingShape)
            for b in [realBuf]:
                b = concatenate((padding, b, padding), axis=1)
        elif yPadding < 0:
            yPadding *= -1
            assert yPadding % 2 == 0
            for b in [realBuf]:
                b = b[  :, yPadding / 2:-yPadding / 2, :]

        ### add 0 x boundary
        self.log.debug('start x-padding')
        self.log.debug(realBuf.shape)
        xPadding = l[2] - s[2]
        if xPadding > 0:
            assert xPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[2] = xPadding / 2
            padding = zeros(paddingShape)
            realBuf = concatenate((padding, realBuf, padding), axis=2)
        elif xPadding < 0:
            xPadding *= -1
            assert xPadding % 2 == 0
            realBuf = realBuf[ :, :, xPadding / 2:-xPadding / 2]

        self.log.debug(realBuf.shape)

        provider.storeExtrapolationProxy(realBuf, flag=flag)

        return realBuf

class geneCombinationGridFieldEigenvector(geneCombinationGridField):
    def __init__(self, levelvector, checkpointProvider,nEV=0, order=1,nField=0,\
                 boundaries=(True, True, True, False, True)):
        self.nEV = nEV
        self.nField = nField
        super(geneCombinationGridFieldEigenvector, self).__init__(levelvector, checkpointProvider, \
                                                                  order, boundaries)

    def getData(self):
        return self.provider.cp.fieldData[self.nEV][self.nField]

class geneCombinationGridFieldIV(geneCombinationGridField):
    def __init__(self,levelvector,checkpointProvider, snapshot=-1,
                 order =1 , nField=0,
                 boundaries = (True,True,True,False,True)):
        self.snapshot = snapshot
        self.nField = nField
        super(geneCombinationGridFieldIV,self).__init__(levelvector,checkpointProvider, order,boundaries)

    def getData(self):
        data = self.provider.cp.fieldData[self.snapshot][self.nField]
        return data


class geneCombinationGridG(geneCombinationGrid):
    '''Here a combination grid using the distribution function is created'''
    nProcesses = 1  # cpu_count()
    #     processPool = Pool(processes=nProcesses)

    def __init__(self, levelvector, provider, sampling_strides = (1,1,1,1,1), take_surpluss=False, *args, **kwargs):
        geneCombinationGrid.__init__(self, levelvector, provider ,*args, **kwargs)
        self.take_surpluss = take_surpluss
        self.sampling_strides = sampling_strides
        # self.log.debug('use ' + str(self.nProcesses) + ' for extrapolation')
        self.changeStrategy('default')

    def changeStrategy(self, name):
        if name == 'regular' or name == 'default':
            self.extrapolateData = self.extrapolateDataRegular
            self.strategy = 'default'
        elif name == 'abs':
            self.extrapolateData = self.extrapolateDataAbs
            self.strategy = 'abs'
        else:
            raise ArgumentError('wrong strategy chosen')

    def injection(self,data_in,full=True):
        """
        this reduces the input data to a smaller size using a full injection scheme. This is necessary in order for
        iterative refinement to work

        :param data_in: That is the input data of shape 2**l+1
        :return: the injected data in from 2**l+1
        """
        if full is not True:
            raise NotImplementedError

        ## sizes
        rlog2 = lambda x:np.rint(np.log2(x)).astype(int)
        s_l = np.array(data_in.shape)[1:].astype(int)
        s_s = 2**np.array(self.levelvec)+1
        self.log.debug("{}\t{}".format(s_l,s_s))
        diff = s_l-s_s
        diff_level = rlog2(s_l-1)-rlog2(s_s-1)

        assert (diff>=0).all()

        ret = data_in.copy()
        ## x and y direction: simple injection -- TODO aliasing might be necessary
        if diff[-1]>0:
            ret = ret[:,:,:,:,:,diff[-1]/2:diff[-1]/2]
        if diff[-2]>0:
            ret = ret[:,:,:,:,diff[-2]/2:diff[-2]/2,:]

        ## z direction
        ## innen
        if diff[-3]>0:
            for l in range(diff_level[-3]):
                ## zentrale Punkte
                buf = 0.5*ret[:,:,:,::2,:,:]
                ## linke Punkte
                buf[:,:,:, 1:,:,:] += 0.25 * ret[:,:,:,1::2,:,:]
                ## rechte Punkte
                buf[:,:,:,:-1,:,:] += 0.25 * ret[:,:,:,1::2,:,:]

                ## periodische Randpunkte
                buf[:,:,:,0,:,1:] += 0.25 * ret[:,:,:,-1,:,:-1]
                buf[:,:,:,-1,:,:-1] += 0.25 * ret[:,:,:,0,:,1:]

                ## echte Randpunkte
                buf[:,:,:,0,:,0]   *= 4./3.
                buf[:,:,:,-1,:,-1] *= 4./3.

                ret = buf

        ## v direction
        if diff[1]>0:
            for l in range(diff_level[1]):
                ##zentrale Punkte
                buf = 0.5*ret[:,:,::2,:,:,:]

                ## links
                buf[:,:,1:,:,:,:] += 0.25*ret[:,:,1::2,:,:,:]

                ## rechts
                buf[:,:,:-1,:,:,:]+= 0.25*ret[:,:,1::2,:,:,:]

                ## boundary
                buf[:,:,0,:,:,:] *= 4./3.
                buf[:,:,-1,:,:,:]*= 4./3.

                ret = buf

        ## mu direction
        ## tricky since they might not be equidistant
        if diff[0]>0:
            if self.provider.get_mu_grid_type()=='clenshaw_curtis':
                ###TODO this is buggy!
                for l in range(diff_level[0]):
                    fine_points=[]
                    for i in range(ret.shape[1]):
                        fine_points.append(self.calcPoint(i,ret.shape[1]))
                    fine_points=np.array(fine_points)

                    center_weight=0.5

                    left_weight =  fine_points[2::2]  - fine_points[1::2]
                    left_weight /= fine_points[2::2]  - fine_points[:-2:2]

                    right_weight= -fine_points[:-2:2] + fine_points[1::2]
                    right_weight/= fine_points[2::2]  - fine_points[:-2:2]

                    buf = center_weight * buf[:,::2,:,:,:,:]
                    buf[1:]+= left_weight * buf[:,1::2,:,:,:,:]
                    buf[:-1]+=right_weight* buf[:,1::2,:,:,:,:]

                    buf[0] /= right_weight[0]/0.5 + 0.5
                    buf[-1]/= left_weight[-1]/0.5 + 0.5

                    ret = buf
            else:
                for l in range(rlog2(diff[0])):
                    ##zentrale Punkte
                    buf = 0.5*ret[:,::2,:,:,:,:]

                    ## links
                    buf[:,1:,:,:,:,:] += 0.25*ret[:,1::2,:,:,:,:]

                    ## rechts
                    buf[:,:-1,:,:,:,:]+= 0.25*ret[:,1::2,:,:,:,:]

                    ## boundary
                    buf[:,0,:,:,:,:] *= 4./3.
                    buf[:,-1,:,:,:,:]*= 4./3.

                    ret = buf

        return ret

    def getExtrapolatedDataSize(self, l):
        '''Might not work, if the cp has not been created yet'''
        return (
            self.provider.cp.getPar('n_spec'), 2 ** l[0] + 1, 2 ** l[1] + 1, 2 ** l[2] + 1, 2 ** l[3] + 1,
            2 ** l[4] + 1)

    def extrapolateData(self, level):
        raise NotImplementedError('this method should not be called')

    def calcPoint(self,i,N):
        """
        calc coordinate positions for clenshaw curtis integration
        """
        return -log(0.5*(cos(i*pi/N)+1))


    def weights(self,i,ncoarse,nfine):
        '''gives the weights for the interpolation'''
        self.log.debug(str(ncoarse)+'   '+str(nfine))
        stride = nfine/ncoarse
        iLeft = i
        iRight = i+1
        iFine = array([iLeft*stride + count+1 for count in range(stride-1)])
        #print iFine
        xLeft = self.calcPoint(iLeft,ncoarse)
        if iRight>=ncoarse:
            xRight = self.calcPoint(iRight*stride*2-1,nfine*2)
        else:
            xRight = self.calcPoint(iRight,ncoarse)
        xInner = self.calcPoint(iFine,nfine)
        return (xRight-xInner)/(xRight-xLeft)

    def mu_interpolation_CC(self,data,l_fine):
        self.log.debug('Start interpolation of the Clenshaw Curtis points in mu direction')
        l_coarse = self.levelvec[0]

        if l_coarse==l_fine:
            return data

        elif l_fine > l_coarse:
            s = array(data.shape)
            stride = 2**(l_fine-l_coarse)
            s[1]=(s[1]-1)*stride+1
            s= tuple(s)
            new_data = zeros(s,dtype=complex)
            new_data[:,::stride,:,:,:,:]=data
            for i in range(2**l_coarse):
                w=self.weights(i,2**l_coarse,2**l_fine)
                self.log.debug(str(w.shape))
                self.log.debug(str(w))
                for j in range(len(w)):
                    if i<2**l_coarse-1:
                        new_data[:,i*stride+1+j,...]=data[:,i,...]*w[j]+data[:,i+1,...]*(1.0-w[j])
                    else:
                        new_data[:,i*stride+1+j,...]=data[:,i,...]*w[j]

            return new_data
        else:
            stride = 2**(l_coarse-l_fine)
            return data[:,::stride,...]




    def getPadding(self):
        buf = self.getData()
        s = 2 ** array(self.levelvec) + 1
        s = list(s)
        s.insert(0, buf.shape[0])
        res = zeros(s, dtype=complex)
        start = array(self.boundaries) * -1 + 1

        self.log.debug(str(s))
        self.log.debug(str(start))

        # #padding
        res[:, start[0]:s[1] - start[0], start[1]:s[2] - start[1], start[2]:s[3] - start[2], start[3]:s[4] - start[3], \
        start[4]:s[5] - start[4]] = buf
        return res

    def checkDataShape(self, data):
        s = list(data.shape)[1:]
        self.log.debug(str(s))
        self.log.debug(str(list(self.gridSizes)))
        assert s == list(self.gridSizes),"{} {}".format(s,self.gridSizes)

    @staticmethod
    def samplingDecorator(extrapolate):
        def new_extrapolate(self,*args,**kwargs):
            buf = extrapolate(self,*args,**kwargs)
            s = self.sampling_strides
            if s==(1,1,1,1,1):
                self.log.info('No hierarchization done: stride is 1')
                return buf

            avg_small = buf[:,::s[0],::s[1],::s[2],::s[3],::s[4]]


            zoom_factor=list(array(buf.shape,dtype=float)/array(avg_small.shape))
            avg_large_real = zoom(avg_small.real,zoom_factor,order=1)
            avg_large_imag = zoom(avg_small.imag,zoom_factor,order=1)
            avg_large=avg_large_real + 1.0j*avg_large_imag

            if self.take_surpluss:
                self.log.info('Hierarchization done: returning surplus')
                return buf-avg_large
            else:
                return avg_large

        return new_extrapolate



class geneCombinationGridIV(geneCombinationGridG):
    def __init__(self, levelvector, provider, snapshot=-1, *args, **kwargs):
        geneCombinationGridG.__init__(self, levelvector, provider, *args, **kwargs)
        self.snapshot = snapshot

    def getData(self):
        data = self.provider.data[self.snapshot]
        self.checkDataShape(data)
        return data

    def downSampleDataRegular(self,level,providerIn=None):
        self.log.debug('start the downsampling of data to {}'.format(level))

        if providerIn == None:
            provider = self.provider
        else:
            provider = providerIn

    @geneCombinationGridG.samplingDecorator
    def extrapolateDataRegular(self, level, providerIn=None,smoothing=False):
        '''TODO needs changes for global runs'''

        self.log.info('Start Extrapolation to '+str(level))

        if providerIn == None:
            provider = self.provider
        else:
            provider = providerIn

        # #check for proxies first...eigentlich im provider
        flag = 'default_snapshot_' + str(self.snapshot)
        flag += '_ord_' + str(self.order)

        buf = provider.getExtrapolationFromProxy(level, flag=flag)

        if buf != None:
            self.log.info('Loaded Extrapolation from Proxy')
            return buf
        #         processPool = Pool(processes=self.nProcesses)
        self.fillData()

        buf = self.getPadding()

        s = buf.shape  # shape of the original array - should be the boundaries
        l = 2 ** array(level) + 1  # size of the new array
        l = list(l)
        l.insert(0, s[0])
        self.log.debug(str(s))
        self.log.debug(str(l))


        ## real space coordinates z,v,w
        spaceLevels = (array(l) * 1.0) / array(s)

        spaceLevels[4] = 1.0
        spaceLevels[5] = 1.0


        ##go over all directions and sample if necessary --> less effort in zoom
        if spaceLevels[2]<1.0:
            local_stride = (s[2]-1)/(l[2]-1)
            buf=buf[:,:,::local_stride,:,:,:]
            spaceLevels[2]=1.0
        if spaceLevels[3]<1.0:
            local_stride = (s[3]-1)/(l[3]-1)
            buf=buf[:,:,:,::local_stride,:,:]
            spaceLevels[3]=1.0

        if self.provider.get_mu_grid_type()=='clenshaw_curtis':
            spaceLevels[1]=1.0
            buf = self.mu_interpolation_CC(buf,level[0])

        self.log.debug(str(buf.max())+'  '+str(buf.real.max())+' '+str(buf.shape))
        #if buf.shape[0]>0:
            #self.log.debug("if there are electrons, that is the max: {}".format(buf[1].max()))
            #imagBuf[i] = zoom(buf[i].imag, spaceLevels[1:], order=self.order)
        if not (spaceLevels == 1.0).all():
            self.log.debug('Start zoom procedure with zoomFactor ' + str(spaceLevels))
            # # local_size = l
            # realBuf=zeros(local_size,dtype=complex)
            # imagBuf=zeros(local_size,dtype=complex)
            realBuf = None
            imagBuf = None
            # self.log.debug(local_size)

            # local_size = list(l)[:4]+list(s)[4:]
            for i in range(buf.shape[0]):
                bu = zoom(buf[i].real, spaceLevels[1:], order=self.order)
                if realBuf is None:
                    realBuf=np.zeros([l[0]]+list(bu.shape))
                realBuf[i]=bu
                bu = zoom(buf[i].imag, spaceLevels[1:], order=self.order)
                if imagBuf is None:
                    imagBuf=np.zeros([l[0]]+list(bu.shape))
                imagBuf[i]=bu


        else:
            realBuf = buf.real
            imagBuf = buf.imag
        self.log.debug(str(realBuf.max()))
        #if realBuf.shape[0]>0:
        #    self.log.debug("if there are electrons, that is the max: {}".format(realBuf[1].max()))
        ### add 0 y boundary
        self.log.debug('Start y Padding')
        yPadding = l[4] - s[4]
        if yPadding > 0:
            assert yPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[4] = yPadding / 2
            padding = zeros(paddingShape)
            for b in [realBuf, imagBuf]:
                b = concatenate((padding, b, padding), axis=4)

        elif yPadding < 0:
            yPadding *= -1
            assert yPadding % 2 == 0
            for b in [realBuf, imagBuf]:
                b = b[:, :, :, :, yPadding / 2:-yPadding / 2, :]

        ### add 0 x boundary
        self.log.debug('start x-padding')
        self.log.debug(realBuf.shape)
        xPadding = l[5] - s[5]
        if xPadding > 0:
            assert xPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[5] = xPadding / 2
            padding = zeros(paddingShape)
            realBuf = concatenate((padding, realBuf, padding), axis=5)
            imagBuf = concatenate((padding, imagBuf, padding), axis=5)
        elif xPadding < 0:
            xPadding *= -1
            assert xPadding % 2 == 0
            realBuf = realBuf[:, :, :, :, :, xPadding / 2:-xPadding / 2]
            imagBuf = imagBuf[:, :, :, :, :, xPadding / 2:-xPadding / 2]

        self.log.debug(realBuf.shape)

        buf = (realBuf + 1.j * imagBuf)
        provider.storeExtrapolationProxy(buf, flag=flag)

        return buf

    @geneCombinationGridG.samplingDecorator
    def extrapolateDataAbs(self, level, providerIn=None,smoothing=False):
        '''TODO needs changes for global runs'''

        self.log.info('Start Extrapolation to '+str(level))
        self.log.info('use absolute values for combination')

        if providerIn == None:
            provider = self.provider
        else:
            provider = providerIn

        # #check for proxies first...eigentlich im provider
        flag = 'abs_snapshot_' + str(self.snapshot)
        flag += '_ord_' + str(self.order)

        buf = provider.getExtrapolationFromProxy(level, flag=flag)

        if buf is None:
                self.log.info('loaded reg snapshot instead of abs')
                reg_flag = 'default_snapshot_'+str(self.snapshot)+'_ord_'+str(self.order)
                buf = provider.getExtrapolationFromProxy(level,flag=reg_flag)
                if buf is not None:
                    return abs(buf)

        if buf != None:
            self.log.info('Loaded Extrapolation from Proxy')
            return buf
        #         processPool = Pool(processes=self.nProcesses)
        self.fillData()

        buf = abs(self.getPadding())

        s = buf.shape  # shape of the original array - should be the boundaries
        l = 2 ** array(level) + 1  # size of the new array
        l = list(l)
        l.insert(0, s[0])
        self.log.debug(str(s))
        self.log.debug(str(l))


        ## real space coordinates z,v,w
        spaceLevels = (array(l) * 1.0) / array(s)

        spaceLevels[4] = 1.0
        spaceLevels[5] = 1.0


        ##go over all directions and sample if necessary --> less effort in zoom
        if spaceLevels[2]<1.0:
            local_stride = (s[2]-1)/(l[2]-1)
            buf=buf[:,:,::local_stride,:,:,:]
            spaceLevels[2]=1.0
        if spaceLevels[3]<1.0:
            local_stride = (s[3]-1)/(l[3]-1)
            buf=buf[:,:,:,::local_stride,:,:]
            spaceLevels[3]=1.0

        if self.provider.get_mu_grid_type()=='clenshaw_curtis':
            spaceLevels[1]=1.0
            buf = abs(self.mu_interpolation_CC(buf,level[0]))

        # self.log.debug(str(buf.max())+'  '+str(buf.real.max())+' '+str(buf.shape))
        #if buf.shape[0]>0:
            #self.log.debug("if there are electrons, that is the max: {}".format(buf[1].max()))
            #imagBuf[i] = zoom(buf[i].imag, spaceLevels[1:], order=self.order)
        if not (spaceLevels == 1.0).all():
            self.log.debug('Start zoom procedure with zoomFactor ' + str(spaceLevels))
            # # local_size = l
            # realBuf=zeros(local_size,dtype=complex)
            # imagBuf=zeros(local_size,dtype=complex)
            realBuf = None
            #imagBuf = None
            # self.log.debug(local_size)

            # local_size = list(l)[:4]+list(s)[4:]
            for i in range(buf.shape[0]):
                bu = zoom(buf[i], spaceLevels[1:], order=self.order)
                if realBuf is None:
                    realBuf=np.zeros([l[0]]+list(bu.shape))
                realBuf[i]=bu
                # bu = zoom(buf[i].imag, spaceLevels[1:], order=self.order)
                # if imagBuf is None:
                #     imagBuf=np.zeros([l[0]]+list(bu.shape))
                # imagBuf[i]=bu


        else:
            realBuf = buf
            #imagBuf = buf.imag
        self.log.debug(str(realBuf.max()))
        #if realBuf.shape[0]>0:
        #    self.log.debug("if there are electrons, that is the max: {}".format(realBuf[1].max()))
        ### add 0 y boundary


        self.log.debug('Start y Padding')
        yPadding = l[4] - s[4]
        if yPadding > 0:
            assert yPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[4] = yPadding / 2
            padding = zeros(paddingShape)
            for b in [realBuf]:
                b = concatenate((padding, b, padding), axis=4)

        elif yPadding < 0:
            yPadding *= -1
            assert yPadding % 2 == 0
            for b in [realBuf]:
                b = b[:, :, :, :, yPadding / 2:-yPadding / 2, :]

        ### add 0 x boundary
        self.log.debug('start x-padding')
        self.log.debug(realBuf.shape)
        xPadding = l[5] - s[5]
        if xPadding > 0:
            assert xPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[5] = xPadding / 2
            padding = zeros(paddingShape)
            realBuf = concatenate((padding, realBuf, padding), axis=5)
            #imagBuf = concatenate((padding, imagBuf, padding), axis=5)
        elif xPadding < 0:
            xPadding *= -1
            assert xPadding % 2 == 0
            realBuf = realBuf[:, :, :, :, :, xPadding / 2:-xPadding / 2]
            #imagBuf = imagBuf[:, :, :, :, :, xPadding / 2:-xPadding / 2]

        self.log.debug(realBuf.shape)

        buf = realBuf
        provider.storeExtrapolationProxy(buf, flag=flag)

        return buf


class geneCombinationGridEigenvalue(geneCombinationGrid):
    def __init__(self, levelvector, provider, nEV=0, *args, **kwargs):
        geneCombinationGrid.__init__(self, levelvector, provider, *args, **kwargs)
        self.nEV = nEV

    def getData(self):  # missingGrids.extend([lmax])
        #return self.provider.EV[self.nEV]
        return self.provider.getEV(self.nEV)

    def extrapolateData(self, level):
        return self.getData()

    def getExtrapolatedDataSize(self, level):
        return (1)


class geneCombinationGridEigenvector(geneCombinationGridG):
    def __init__(self, levelvector, provider, nEV=0, *args, **kwargs):
        geneCombinationGridG.__init__(self, levelvector, provider, *args, **kwargs)
        self.nEV = nEV

    def getData(self):
        data = self.provider.data[self.nEV]
        self.checkDataShape(data)
        return data

    @geneCombinationGridG.samplingDecorator
    def extrapolateDataRegular(self, level, providerIn=None,data_override=None,no_proxy=False):
        '''changes for global runs'''

        if providerIn == None:
            provider = self.provider
        else:
            provider = providerIn

        # #check for proxies first...eigentlich im provider
        flag = 'default_ev_' + str(self.nEV)
        flag += '_ord_' + str(self.order)

        if not no_proxy:
                buf = provider.getExtrapolationFromProxy(level, flag=flag)
        else:
                buf = None

        if (buf is not None) and (data_override is None) and not no_proxy:
            self.log.info('Loaded Extrapolation from Proxy')
            return buf
        self.fillData()

        if data_override is not None:
            buf = data_override
        else:
            buf = self.getPadding()

        s = buf.shape  # shape of the original array - should be the boundaries
        l = 2 ** array(level) + 1  # size of the new array
        l = list(l)
        l.insert(0, s[0])
        self.log.debug(str(s))
        self.log.debug(str(l))


        ## real space coordinates z,v,w
        spaceLevels = (array(l) * 1.0) / array(s)

        spaceLevels[4] = 1.0
        spaceLevels[5] = 1.0

        if self.provider.get_mu_grid_type()=='clenshaw_curtis':
            spaceLevels[1]=1.0
            buf = self.mu_interpolation_CC(buf,level[0])

        self.log.info('Start zoom procedure with zoomFactor ' + str(spaceLevels))

        if not (spaceLevels == 1.0).all():
            realBuf = zoom(buf.real, spaceLevels, order=self.order)
            imagBuf = zoom(buf.imag, spaceLevels, order=self.order)
        else:
            realBuf = buf.real
            imagBuf = buf.imag

        ### add 0 y boundary
        self.log.debug('Start y Padding')
        yPadding = l[4] - s[4]
        if yPadding > 0:
            assert yPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[4] = yPadding / 2
            padding = zeros(paddingShape)
            for b in [realBuf, imagBuf]:
                b = concatenate((padding, b, padding), axis=4)
        elif yPadding < 0:
            yPadding *= -1
            assert yPadding % 2 == 0
            for b in [realBuf, imagBuf]:
                b = b[:, :, :, :, yPadding / 2:-yPadding / 2, :]

        ### add 0 x boundary
        self.log.debug('start x-padding')
        self.log.debug(realBuf.shape)
        xPadding = l[5] - s[5]
        if xPadding > 0:
            assert xPadding % 2 == 0
            paddingShape = l[:]
            paddingShape[5] = xPadding / 2
            padding = zeros(paddingShape)
            realBuf = concatenate((padding, realBuf, padding), axis=5)
            imagBuf = concatenate((padding, imagBuf, padding), axis=5)
        elif xPadding < 0:
            xPadding *= -1
            assert xPadding % 2 == 0
            realBuf = realBuf[:, :, :, :, :, xPadding / 2:-xPadding / 2]
            imagBuf = imagBuf[:, :, :, :, :, xPadding / 2:-xPadding / 2]

        self.log.debug(realBuf.shape)

        buf = (realBuf + 1.j * imagBuf)
        if not no_proxy:
            provider.storeExtrapolationProxy(buf, flag=flag)

        return buf

    def extrapolateDataAbs(self, level, providerIn=None):
        '''changes for global runs'''

        if providerIn is None:
            provider = self.provider
        else:
            provider = providerIn
        # #here sollten die Daten schon auf ein 2**n+1 Gitter gebracht werden
        # #Nullen anfuegen

        # #check for proxies first...eigentlich im provider
        flag = 'abs_ev_' + str(self.nEV)
        flag += '_ord_' + str(self.order)
        buf = provider.getExtrapolationFromProxy(level, flag=flag)

        if buf != None:
            self.log.info('Loaded Extrapolation from Proxy')
            return buf

        self.fillData()

        buf = self.getPadding()
        processPool = Pool(processes=self.nProcesses)

        s = buf.shape  # shape of the original array - should be the boundaries
        l = 2 ** array(level) + 1  # size of the new array
        l = list(l)
        l.insert(0, s[0])
        self.log.debug(str(s))
        self.log.debug(str(l))

        yStart = -(l[4] - s[4]) / 2
        yEnd = yStart + l[4]  # not including this value

        xStart = -(l[5] - s[5]) / 2
        xEnd = xStart + l[5]  # not including this value

        self.log.debug(str(xStart) + ' ' + str(xEnd) + ' ' + str(yStart) + ' ' + str(yEnd))
        # #new coordinates
        coords = mgrid[0:s[0], 0:s[1] - 1:l[1] * 1j, 0:s[2] - 1:l[2] * 1j, 0:s[3] - 1:l[3] * 1j, \
                 # # x and ydirection -- fourier!
                 yStart:yEnd, xStart:xEnd]

        coord = coords.reshape((6, -1))
        self.log.debug(str(coord.shape))
        #         self.log.debug(str(coord))
        self.log.debug('Coordinates:')
        for i in range(6):
            self.log.debug(str(unique(coord[i])))

        chunkSize = ceil(coord.shape[1] / (self.nProcesses * 1.0))
        realTuples = zip([abs(buf)] * self.nProcesses,
                         [coord[:, i * chunkSize:(i + 1) * chunkSize:] for i in range(self.nProcesses)])

        self.log.info('map_coordinates of grid' + str(s) + ' using ' + str(
            self.processPool._processes) + ' processes for extrapolation')

        #         real = self.processPool.map(mapCoordWrapper, realTuples)
        real = processPool.map(mapCoordWrapper, realTuples, self.order)
        real = concatenate(real)

        self.log.debug(str(real.shape))
        self.log.debug(str(l))

        buf = real.reshape(l)
        provider.storeExtrapolationProxy(buf, flag=flag)

        return buf


class EVOpticomCombigridBase(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def applyOperator(self, level, *args, **kwargs):
        return

    @abc.abstractmethod
    def setOperator(self, *args, **kwargs):
        return

    def getProvider(self,level):
        """
        This method applies the GENE linear operator on an existing GENE checkpoint
        :param level: Level of the larger operator
        :return: full grid data of large operator :raise IOError:
        """
        assert hasattr(self, 'operatorGeneEnv')
        geneEnv =self.operatorGeneEnv

        if self.tag is None:
            pathToCheckpoint = self.provider.getDir() + 'restart_' + str(level) + '/'
        else:
            pathToCheckpoint = self.provider.getDir() + 'restart_' + str(level) + '_'+self.tag+'/'

        self.log.debug(pathToCheckpoint)
        try:
            cpWithL = checkpoint2Local(pathToCheckpoint)
            if not cpWithL.hasParFile:
                raise IOError
        except IOError:
            self.log.debug(pathToCheckpoint)

            # #copy checkpoint to geneEnv (has to be the corresponding geneEnv
            self.log.debug(geneEnv.diagDir + '\t' + self.provider.getDir())
            # # here we make surre to get a geneEnv, which we can determine ourselfes
            geneEnv.update(pathToCheckpoint)
            shutil.copy(pathToCheckpoint + 'checkpoint_extra', pathToCheckpoint + 'checkpoint')
            # #copy the parameters? --> might destroy existing stuff -- just copy checkpoint and be sure to run the
            # right geneEnv

            geneEnv.setGridSizeWithLevel(level)
            geneEnv.parFileIO.updateParameter('read_checkpoint', 'T')
            geneEnv.parFileIO.updateParameter('which_ev', "'rhs_only'")
            geneEnv.parFileIO.updateParameter('comp_type', "'EV'")
	    #geneEnv.parFileIO.updateParamater('comp_type',"'EV'")
            geneEnv.runGENE()
            self.log.debug('\n' + geneEnv.getStdout())
            cpWithL = checkpoint2Local(pathToCheckpoint)


        providerWithL = DataProviderCP(cpWithL, useMapProxies=True)
        self.log.debug('created checkpoint of outcome of matrix vector operation')

        #         providerWithL.fillData()

        return providerWithL


class geneEVOpticomGridEigenvector(geneCombinationGridEigenvector, EVOpticomCombigridBase):
    # #constructor necessary?

    def setOperator(self, geneEnv,tag=None):
        self.operatorGeneEnv = GeneInterface.geneEnvironment2(\
            cp.copy(geneEnv.genePath),cp.copy(geneEnv.problemFolder),\
            cp.copy(geneEnv.diagDir))
        self.operatorGeneEnv.runCommand = cp.copy(geneEnv.runCommand)
        self.tag =tag

    def applyOperator(self, level, smoothing=False):
        #self.storeExtrapolatedDataForRestart(level,tag=self.tag)
        self.storeCheckpointForRestart(level,tag=self.tag)
        providerWithL = self.getProvider(level)
        bufferGrid = geneCombinationGridEigenvector(level, providerWithL, order=self.order)
        bufferGrid.fillData()

        if smoothing:
        	print '-------smooothing------------'
        	print providerWithL.cp.eigenvectors[0].shape
        	#d=providerWithL.cp.eigenvectors[0] 
        	d = bufferGrid.extrapolateData(bufferGrid.levelvec)
        	#smoothData=np.zeros(d.shape,dtype=complex)
        	#smoothData[:,:,:,1:-1,:,:]=0.25*d[:,:,:,:-2,:,:]+0.5*d[:,:,:,1:-1,:,:]+0.25*d[:,:,:,2:,:,:]
        	##left boundary
        	#smoothData[:,:,:,0,:,:]=0.5*(d[:,:,:,0,:,:]+d[:,:,:,1,:,:])
        	#smoothData[:,:,:,-1,:,:]=0.5*(d[:,:,:,-2,:,:]+d[:,:,:,-1,:,:])
        
        	#smoothData*=0
        	from scipy.signal import savgol_filter
        	smoothData = savgol_filter(d.real,3,1,axis=-3) + 1.j*savgol_filter(d.imag,3,1,axis=-3)
        	smoothData = savgol_filter(smoothData.real,3,1,axis=-4) + 1.j*savgol_filter(smoothData.imag,3,1,axis=-4)
        	smoothData = savgol_filter(smoothData.real,3,1,axis=-5) + 1.j*savgol_filter(smoothData.imag,3,1,axis=-5)
        
        
        	#providerWithL.cp.eigenvectors[0] = smoothData*0
        	#bufferGrid.provider.fillDataFromEV()
        
        	return bufferGrid.extrapolateData(level,no_proxy=True,data_override=smoothData)
        
        else:	
                	return bufferGrid.extrapolateData(level)

class geneEVOpticomGridFieldEigenvector(geneCombinationGridFieldEigenvector, EVOpticomCombigridBase):
    def setOperator(self, geneEnv,tag=None):
        self.operatorGeneEnv = GeneInterface.geneEnvironment2(\
            cp.copy(geneEnv.genePath),cp.copy(geneEnv.problemFolder),\
            cp.copy(geneEnv.diagDir))
        self.tag =tag

    def applyOperator(self, level,smoothing=False):
        # print type(self)
        self.storeExtrapolatedDataForRestart(level,tag=self.tag)
        providerWithL = self.getProvider(level)
        bufferGrid = geneCombinationGridFieldEigenvector(level, providerWithL, order=self.order)
        bufferGrid.fillData()
        return bufferGrid.extrapolateData(level)


class combiGridDummy(combiGrid, EVOpticomCombigridBase):
    def __init__(self, levelvector, data, boundaries=(True, True)):
        combiGrid.__init__(self, levelvector, boundaries)
        self.orig = data

    def fillData(self):
        self.data = self.orig.copy()

    def getData(self):
        return self.data

    def extrapolateData(self, level):
        return self.data

    def setOperator(self, A):
        self.A = A

    def applyOperator(self, level,smoothing=False):
        return dot(self.A, self.data)


class IterativeRefinementCombiGridBase(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def solveSystem(self, *args, **kwargs):
        return

    @abc.abstractmethod
    def setSystem(self, *args, **kwargs):
        return


class  IterativeRefinementEVOpticomGridEigenvector(geneEVOpticomGridEigenvector,IterativeRefinementCombiGridBase):

    def setSystem(self, geneEnv,tag=None):
        self.tag=tag
        self.iterativeOperatorGeneEnv = GeneInterface.geneEnvironment2(\
            cp.copy(geneEnv.genePath),cp.copy(geneEnv.problemFolder),\
            cp.copy(geneEnv.diagDir))
        self.iterativeOperatorGeneEnv.runCommand = cp.copy(geneEnv.runCommand)

    def solveSystem(self, extrapolatedData,ev_shift,hash_id,tag=None,a_tol = '1e-7',r_tol='1e-10'):
        """
        This method gets the rhs and solve the system
        :param extrapolatedData: rhs for which to be solved
        :return: extrapolated data
        """
        self.a_tol=a_tol
        self.r_tol=r_tol
        if tag is not None:
            self.tag=tag
        self.storeDataInAppropriateSize(extrapolatedData,tag=hash_id+'_'+('' if self.tag is None else self.tag))
        self.ev_shift=ev_shift
        providerDeltaG = self.getIterativeProvider(hash_id)
        solutionGrid = geneCombinationGridEigenvector(self.levelvec,providerDeltaG)
        solutionGrid.fillData()
        l = np.rint(log2(np.array(extrapolatedData.shape)[1:]-np.array([1,1,1,1,1])))

        return solutionGrid.extrapolateData(l)

    def getIterativeProvider(self,norm_hash):
        """
        This method solves (L-lambda I)g
        :param level: Level of the larger operator
        :return: full grid data of large operator :raise IOError:
        """
        assert hasattr(self, 'operatorGeneEnv')
        geneEnv =self.iterativeOperatorGeneEnv

        self.file_tag = 'iter_ref_'

        if self.tag is None:
            pathToCheckpoint = self.provider.getDir() + self.file_tag + norm_hash + '/'
        else:
            pathToCheckpoint = self.provider.getDir() + self.file_tag + norm_hash + '_'+self.tag+'/'

        self.log.debug(pathToCheckpoint)
        try:
            cpWithL = checkpoint2Local(pathToCheckpoint)
            if not cpWithL.hasParFile:
                raise IOError
        except IOError:
            self.log.debug(pathToCheckpoint)

            # #copy checkpoint to geneEnv (has to be the corresponding geneEnv
            self.log.debug(geneEnv.diagDir + '\t' + self.provider.getDir())
            # # here we make surre to get a geneEnv, which we can determine ourselfes
            geneEnv.update(pathToCheckpoint)
            shutil.copy(pathToCheckpoint + 'checkpoint_iter_ref', pathToCheckpoint + 'checkpoint')
            # #copy the parameters? --> might destroy existing stuff -- just copy checkpoint and be sure to run the
            # right geneEnv

            geneEnv.setGridSizeWithLevel(self.levelvec)
            geneEnv.parFileIO.updateParameter('read_checkpoint', 'T')
            geneEnv.parFileIO.updateParameter('which_ev', "'solve_system'")
            #geneEnv.parFileIO.updateParameter('my_ev_shift','('+str(self.ev_shift.real)+','+str(self.ev_shift.imag)+')')
            geneEnv.parFileIO.updateParameter('ev_shift_ref','('+str(self.ev_shift.real)+','+str(self.ev_shift.imag)+')')

            geneEnv.runGENE(commandLineParm=\
                            '-ksp_view -sub_pc_type lu -ksp_atol {} -ksp_rtol {} -ksp_converged_reason -pc_type asm -pc_asm_overlap 3 -ksp_monitor_true_residual'.format(self.a_tol,self.r_tol))
            ##ksp_monitor_true_residual
            self.log.debug('\n' + geneEnv.getStdout())
            cpWithL = checkpoint2Local(pathToCheckpoint)


        providerWithL = DataProviderCP(cpWithL, useMapProxies=True)
        self.log.debug('created checkpoint of outcome of solving linear system')

        #         providerWithL.fillData()

        return providerWithL

class combiGridDummyMatrix(combiGrid, EVOpticomCombigridBase,IterativeRefinementCombiGridBase):
    def __init__(self, levelvector, data, boundaries=None):
        if boundaries is None:
            boundaries=tuple((np.ones(len(levelvector))).astype(bool))
        combiGrid.__init__(self, levelvector, boundaries)
        self.orig = data

    def fillData(self):
        self.data = self.orig.copy()

    def getData(self):
        return self.data

    def extrapolateData(self, level):
        l=array(level)
        s = array(self.levelvec)
        buf_in = self.data.reshape(self.gridSizes)
        self.log.debug(l)
        self.log.debug(s)

        diff_level=l-s
        # b = aYrray(self.boundaries).astype(int)*2-1
        b = 1.0
        zoom_factor=(2**l+b)/(2**s+b)
        self.log.debug(zoom_factor)

        # padding
        buf = zeros(2**s+1,dtype=buf_in.dtype)
        bound=(array(self.boundaries).astype(int)-1)*-1
        slices=[slice(bound[i],2**s[i]+1-bound[i],1) for i in range(self.getDim())]
        buf[slices]=buf_in

        if self.data.dtype ==complex:
            real = zoom(buf.real,zoom_factor,order=1)
            imag = zoom(buf.imag,zoom_factor,order=1)
            self.log.debug(real.shape)
            ret= (real+1j*imag)
        else:
            self.log.debug(buf.shape)
            ret =  zoom(buf,zoom_factor,order=1)
            self.log.debug(ret.shape)
        return ret



    def setOperator(self, getA):
        self.getA = getA

    def applyOperator(self, level,smoothing=False):
        A =self.getA(level)
        self.log.debug(level)
        self.log.debug(A.shape)
        buf_in = self.extrapolateData(level)
        s=array(buf_in.shape)
        bound=(array(self.boundaries).astype(int)-1)*-1
        slices=[slice(bound[i],s[i]-bound[i],1) for i in range(self.getDim())]
        self.log.debug(slices)
        buf=buf_in[slices]
        self.log.debug(buf.shape)
        self.log.debug(s)
        ret= dot(A, buf.flatten()).reshape(buf.shape)
        buf=zeros(s,dtype=ret.dtype)
        buf[slices]=ret
        self.log.debug(buf.shape)
        return buf.flatten()

    def solveSystem(self,r,lamb):
        self.log.debug(r.shape)
        A=self.getA(self.levelvec)
        b = -(array(self.boundaries).astype(int)-1)
        slices = [slice(b[i],r.shape[i]-b[i],1) for i in range(len(r.shape))]
        self.log.debug(slices)
        residual = r[slices]
        self.log.debug(residual.shape)
        self.log.debug(A.shape)
        buf = np.linalg.solve(A-lamb*np.eye(A.shape[0]),residual.flatten()).reshape(residual.shape)
        self.log.debug(buf.shape)
        erg = zeros(r.shape,dtype=buf.dtype)
        self.log.debug(erg[slices].shape)
        erg[slices]=buf
        self.log.debug(erg.shape)
        return erg

    @staticmethod
    def injection(x,lmax,s):
        diff = array(lmax)-array(s)
        for d in range(len(s)):
            for l in range(diff[d]):
                slices=[slice(0,x.shape[i],1) for i in range(len(s))]
                slices[d]=slice(1,x.shape[d],2)
                buf=0.5*x[slices]
                slices[d]=slice(0,x.shape[d]-1,2)

                buf+=0.25*x[slices]
                slices[d]=slice(2,x.shape[d],2)
                buf+=0.25*x[slices]
                x=buf
        return x

    def matVecSystem(self,level,r,lamb):
        s=array(r.shape)
        l=2**array(level)+1
        self.log.debug(level)
        self.log.debug(s)
        stride=(s-1)/(2**array(level))
        self.log.debug(stride)
        b=array(self.boundaries).astype(int)*2-1
        b2=-(array(self.boundaries).astype(int)-1)
        slices = [slice(0,s[i],stride[i]) for i in range(len(s))]
        slices2=[slice(b2[i],l[i]-b2[i],1) for i in range(len(s))]
        self.log.debug(slices)
        self.log.debug(slices2)
        buf=r[slices]
        # print buf
        self.log.debug(buf.shape)
        buf=buf[slices2]

        A=self.getA(level).astype(complex)
        self.log.debug(A.shape)
        self.log.debug((A-lamb*np.eye(A.shape[0])).shape)
        self.log.debug(buf.shape)
        buf = np.dot(A-lamb*np.eye(A.shape[0]),buf.flatten()).reshape(2**array(level)+b)
        # print '--------\n',buf
        # print buf.real
        ## pad
        buf2 = zeros(2**array(level)+1,dtype=buf.dtype)

        buf2[slices2]=buf
        ## extrapolate

        zoom_factor=s/(l.astype(float))
        self.log.debug(zoom_factor)

        if self.data.dtype ==complex:
            real = zoom(buf2.real,zoom_factor,order=1)
            imag = zoom(buf2.imag,zoom_factor,order=1)
            ret= (real+1j*imag)
        elif self.data.dtype==float:
            ret= zoom(buf2.real,zoom_factor,order=1)
        else:
            raise NotImplementedError('Please Implement this for this data')

        return ret


    def setSystem(self):
        pass

