##from http://bkanuka.com/articles/native-latex-plots
from numpy import sqrt
from subprocess import check_output
from matplotlib.backends.backend_pdf import PdfPages
import logging
import matplotlib2tikz


class pgf_plotter(object):
    pgf_with_latex = {  # setup matplotlib to use latex for output
                        "pgf.texsystem": "pdflatex",  # change this if using xetex or lautex
                        "text.usetex": True,  # use LaTeX to write all text
                        "font.family": "serif",
                        "font.serif": [],  # blank entries should cause plots to inherit fonts from the document
                        "font.sans-serif": [],
                        "font.monospace": [],
                        "axes.labelsize": 10,  # LaTeX default is 10pt font.
                        "text.fontsize": 10,
                        "legend.fontsize": 8,  # Make the legend/label fonts a little smaller
                        "xtick.labelsize": 8,
                        "ytick.labelsize": 8,
                        "lines.markersize": 4,
                        "lines.markeredgewidth": 0.3,
                        "figure.figsize": 1.0,  # default fig size of 0.9 textwidth
                        "pgf.preamble": [
                            r"\usepackage[utf8x]{inputenc}",  # use utf8 fonts becasue your computer can handle it :)
                            r"\usepackage[T1]{fontenc}",  # plots will be generated using this preamble
                        ]
    }

    tikz_param = {
            "font_size":10.0
            }

    golden_mean = (sqrt(5.0) - 1.0) / 2.0

    def __init__(self, matplotlib, textwidth=418.25):
        # Aesthetic ratio (you could change this)
        self.textwidth = textwidth
        # self.plt = matplotlib
        self.option_dict = self.pgf_with_latex
        self.option_dict['figure.figsize'] = self._figsize(1.0)

        self.update_rc(matplotlib)
        self.mpl = matplotlib


    def _figsize(self, scale, height_ratio=golden_mean, fig_h=None, scale_height=1.0):
        # fig_width_pt = 418                         # Get this from LaTeX using \the\textwidth
        # inches_per_pt = 1.0/90.#1.0/72.27                       # Convert pt to inch
        inches_per_pt = 1. / 72.27

        fig_width_pt = self.textwidth

        fig_width = fig_width_pt * inches_per_pt * scale  # width in inches
        if fig_h is None:
            fig_height = self.textwidth * inches_per_pt * height_ratio * scale_height  # height in inches
        else:
            fig_height = fig_width_pt * fig_h * inches_per_pt

        # fig_height = fig_width_pt*0.6*inches_per_pt              # height in inches
        fig_size = [fig_width, fig_height]
        return fig_size

    def get_figsize(self, scale, **kwargs):
        return self._figsize(scale, **kwargs)


    def update_rc(self, plt):
        plt.rcParams.update(self.option_dict)

    def figure(self, fig_command, scale=1.0, y_scale=1.0, **kwargs):
        return fig_command(figsize=self.get_figsize(scale, scale_height=y_scale), **kwargs)


    def savefigure(self, fig, path_in, comment=''):
        path = path_in
        # fig.savefig(path+'.pdf')
        #with PdfPages(path+'.pdf') as p:
        p = PdfPages(path + '.pdf')

        p.savefig(fig)
        d = p.infodict()
        d['Keywords'] = self.s_repo_info + '\n' + ''.join(self.git_info) + '\n' + comment
        d['Author'] = self.author
        p.close()

        try:
            fig.savefig(path + '.pgf')
        except ValueError:
            logging.warn('pgf backend not working')
            pass
        #matplotlib2tikz.save(path+'.tikz',figureheight = '\\figureheight', figurewidth = '\\figurewidth',textsize =self.tikz_param['font_size'] )

        try:
            matplotlib2tikz.save(path+'.tikz',figureheight = '\\figureheight', figurewidth = '\\figurewidth',textsize =self.tikz_param['font_size'] )
        except :
            logging.warn('matplotlib2tikz did not produce output')
        #matplotlib2tikz.save(path+'.tikz')


